

(define (order-2-positions first last)
  ; swap them, if needed
  (if (not (container.find-order (car first) (cadr first)
				 (car last) (cadr last)))
      (let ((car-first (car first))
	    (cdr-first (cdr first))
	    (car-last (car last))
	    (cdr-last (cdr last)))
	(set-car! first car-last)
	(set-cdr! first cdr-last)
	(set-car! last car-first)
	(set-cdr! last cdr-first))))

  


(define (copy-sel)
  (let* ((first (xword-window.get-mark-position xw))
	 (last (xword-window.get-cursor-position xw)))
    (order-2-positions first last)
    (xword-window.copy-to-cut-slot xw
				   (car first) (cadr first)
				   (car last) (cadr last))))


(define (cut-sel)
  (copy-sel)
  (clear-selected-area))



(define (paste-sel)
  (xword-window.get-primary-targets xw))


(define (copy-xcb)
  (let* ((first (xword-window.get-mark-position xw))
	 (last (xword-window.get-cursor-position xw))
	 (str ""))

    (order-2-positions first last)

    (do ((cur (car first) (container.next-node cur)))
	((eqv? cur (container.next-node (car last))) #t)
      (set! str (string-append
		 str
		 (container.get-leaf-ascii cur))))
    (xcutbuffer.set xcb 0 str)))


(define (paste-xcb)
  (let* ((str (xcutbuffer.get xcb 0))
	 (len (string-length str)))
    (do ((i 0 (+ i 1)))
	((>= i len) #t)
      (xword-window.insert-character xw (string-ref str i) cur-attr)))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))



(define (clear-selected-area)
  (let* ((mark (xword-window.get-mark-position xw))
	 (mark-attr (container.get-tmp-attributes (car mark))))
    (if (not (eqv? mark-attr 0))
	(clear-point-to-mark)
	#f)))

(define (clear-point-to-mark)
  (let* ((mark (xword-window.get-mark-position xw))
	 (cur (xword-window.get-cursor-position xw))
	 (mark-con (car mark))
	 (mark-attr (container.get-tmp-attributes mark-con))
	 (ret #f))
    (order-2-positions mark cur)
    (xword-window.set-mark-position xw (car mark) (cadr mark))
    (xword-window.set-cursor-position xw (car cur) (cadr cur))
    (do ()
	((let* ((cur (container.hop-back-position (car cur) (cadr cur)))
		(mark (container.hop-back-position (car mark) (cadr mark))))
	   (and (eqv? (car cur) (car mark))
		(eqv? (cadr cur) (cadr mark))))
	 #t)
      (xword-window.delete-character xw)
      (set! ret #t)
      (set! mark (xword-window.get-mark-position xw))
      (set! cur (xword-window.get-cursor-position xw)))
    (xword-window.unhighlight xw)
    (xword-window.scroll-to-cursor xw)
    (xword-window.redraw xw doc)
    ret))


(define (set-selected-area-text-attributes attrs)
  (let* ((first (xword-window.get-mark-position xw))
	 (last (xword-window.get-cursor-position xw))
	 (tmp-attr (container.get-tmp-attributes (car first))))
    (if (not (eqv? tmp-attr 0))
	(begin
	  (order-2-positions first last)
	  (do ((cur (car first) (container.next-node cur)))
	      ((eqv? cur (container.next-node (car last))) #t)
	    (if (string=? (container.get-type cur) "WordSegment")
		(container.set-attributes cur attrs)))
	  (xword-window.rewrap xw (car first) (car last))))))
