;;;;;;;;;
;; save
;;;;;;;;;

(define (save-ascii-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-save #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-save #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-save-ascii widget client-data call-data))
		      handler)))

    (cond ((eq? can-save #f)
	   (display "can't save ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline)))))

(define (actually-save-ascii widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "w"))))

    (if (eq? fp #f)
	(begin
	  (display "can't save file.")
	  (newline)
	  (error "can't save.")))

    ;(display "file name is ") (display file-name) (newline)

    (do ((con (container.first-lowest-container doc)
	      (container.next-node con)))
	((eqv? con #f) #t)
      (let ((s (container.get-leaf-ascii con)))
	(display s fp)))

    (close-output-port fp)))
	


(define (save-ascii-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (save-ascii doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) save-ascii-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) save-ascii-fsb-cancel (list #f)))


;;;;;;;;;
;; load
;;;;;;;;;

(define (load-ascii-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-load #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-load #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-load-ascii widget client-data call-data))
		      handler)))

    (cond ((eq? can-load #f)
	   (display "can't load ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline))
	  (else
	   (set! doc blah)))))

(define (actually-load-ascii widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "r"))))

    (if (eq? fp #f)
	(begin
	  (display "no such file")
	  (newline)
	  (error "no such file")))

    ;(display "file name is ") (display file-name) (newline)
    ;; free old document here...

    (let* ((doc-style (car doc-styles))
	   (page-style (car page-styles))
	   (column-style (car column-styles))
	   (line-style (car line-styles-low))
	   (word-style (car word-styles))
	   (wordsegment-style (car wordsegment-styles))
	   (new-doc (document.new doc-style)))

      (container.insert-child (container.last-lowest-container new-doc) 0
			      (page.new page-style))

      (container.insert-child (container.last-lowest-container new-doc) 0
			      (container.new column-style))
      
      (container.insert-child (container.last-lowest-container new-doc) 0
			      (container.new line-style))
      
      (do ((parent (container.last-lowest-container new-doc))
	   (index 0 (+ index 1))
	   (input-line (read-line fp 'concat) (read-line fp 'concat)))
	  ;
	  ((eof-object? input-line) #t)
	;
	(let* ((line-length (string-length input-line))
	       (new-word (container.new word-style))
	       (new-wordsegment (wordsegment.new wordsegment-style)))
	  (container.set-leaf-ascii new-wordsegment input-line)
	  (container.insert-child new-word 0 new-wordsegment)
	  (container.insert-child parent index new-word)))

      (close-input-port fp)

      (xword-window.set-doc xw new-doc)

      (xword-window.set-cursor-position
       xw
       (container.first-lowest-container new-doc)
       0)

      (xword-window.rewrap xw
			   (container.first-lowest-container new-doc)
			   (container.last-lowest-container new-doc))

      (xword-window.scroll-to-cursor xw)
      (xword-window.redraw xw new-doc)

      new-doc)))


(define (load-ascii-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (load-ascii doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) load-ascii-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) load-ascii-fsb-cancel (list #f)))


;;;;;;;;;;;
;; insert
;;;;;;;;;;;

(define (insert-ascii-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-insert #t)
	(handler
	 (lambda (key args . rest)
	   (set! can-insert #f)
	   key))
	(blah (catch #t
		     (lambda ()
		       (actually-insert-ascii widget client-data call-data))
		     handler)))
    
    (cond ((eq? can-insert #f)
	   (display "can't insert ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline)))))


(define (actually-insert-ascii widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "rb"))))

    (if (eq? fp #f)
	(begin
	  (display "no such file.")
	  (newline)
	  (error "no such file")))

    (let* ((doc-style (car doc-styles))
	   (page-style (car page-styles))
	   (column-style (car column-styles))
	   (line-style (car line-styles-low))
	   (word-style (car word-styles))
	   (wordsegment-style cur-attr))

      (do ((parent (container.get-parent
		    (container.get-parent
		     (car (xword-window.get-cursor-position xw)))))
	   (index 0 (+ index 1))
	   (input-line (read-line fp 'concat) (read-line fp 'concat)))
	  ;
	  ((eof-object? input-line) #t)
	;
	(let* ((line-length (string-length input-line))
	       (new-word (container.new word-style))
	       (new-wordsegment (wordsegment.new wordsegment-style)))
	  (container.set-leaf-ascii new-wordsegment input-line)
	  (container.insert-child new-word 0 new-wordsegment)
	  (container.insert-child parent index new-word)))
      
      (close-input-port fp)
      
      (xword-window.set-cursor-position
       xw
       (container.first-lowest-container doc)
       0)
      
      (xword-window.rewrap xw
			   (container.first-lowest-container doc)
			   (container.last-lowest-container doc))
      
      (xword-window.scroll-to-cursor xw)
      (xword-window.redraw xw doc)
      )))


(define (insert-ascii-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))

(define (insert-ascii doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) insert-ascii-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback)
		   insert-ascii-fsb-cancel (list #f)))
