;;;;;;;;;
;; save
;;;;;;;;;

(define (save-html-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-save #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-save #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-save-html widget client-data call-data))
		      handler)))

    (cond ((eq? can-save #f)
	   (display "can't save ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline)))))

(define (actually-save-html widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "w"))))

    (if (eq? fp #f)
	(begin
	  (display "can't save file.")
	  (newline)
	  (error "can't save.")))

    ;(display "file name is ") (display file-name) (newline)

    (do ((con (container.first-lowest-container doc)
	      (container.next-node con)))
	((eqv? con #f) #t)
      (let ((s (container.get-leaf-html con)))
	(display s fp)))

    (close-output-port fp)))
	


(define (save-html-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (save-html doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) save-html-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) save-html-fsb-cancel (list #f)))


;;;;;;;;;
;; load
;;;;;;;;;

(define (load-html-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-load #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-load #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-load-html widget client-data call-data))
		      handler)))

    (cond ((eq? can-load #f)
	   (display "can't load ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline))
	  (else
	   (set! doc blah)))))

(define (actually-load-html widget client-data call-data)
;(define (load-html-fsb-ok widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "r"))))

    (if (eq? fp #f)
	(begin
	  (display "no such file")
	  (newline)
	  (error "no such file")))

    ;(display "file name is ") (display file-name) (newline)
    ;; free old document here...

    (let* ((doc-style (car doc-styles))
	   (page-style (car page-styles))
	   (column-style (car column-styles))
	   (line-style (car line-styles-low))
	   (word-style (car word-styles))
	   (wordsegment-style (car wordsegment-styles))
	   (tag-style (car tag-styles))
	   (new-doc (document.new doc-style)))

      (container.insert-child (container.last-lowest-container new-doc) 0
			      (page.new page-style))

      (container.insert-child (container.last-lowest-container new-doc) 0
			      (container.new column-style))
      
      (container.insert-child (container.last-lowest-container new-doc) 0
			      (container.new line-style))
      


      (let* ((done #f))
	(do ((parent (container.last-lowest-container new-doc))
	     (index 0)
	     (in (read-delimited "< " fp 'split)
		 (read-delimited "< " fp 'split)))
	    ((or (eqv? done #t)
		 (eof-object? (car in)))
	     #t)
	  ; did we hit a space character?
	  (if (eqv? (cdr in) #\Space)
	      (begin (set-car! in (string-append (car in) " "))
		     (set-cdr! in #t)))
	  ; handle any plain text that came through
	  (if (> (string-length (car in)) 0)
	      (let* ((new-word (container.new word-style))
		     (new-wordsegment (wordsegment.new wordsegment-style)))
		(container.set-leaf-html new-wordsegment (car in))
		(container.insert-child new-word 0 new-wordsegment)
		(container.insert-child parent index new-word)
		(set! index (+ index 1))))
	  ; did we hit a tag or eof?
	  (cond ((eof-object? (cdr in)) ; eof
		 (set! done #t))
		((eqv? (cdr in) #\<) ; tag
		 (let* ((in-tag (read-delimited ">" fp 'split))
			(new-word (container.new word-style))
			(new-tag (tag.new tag-style)))
		   (container.set-leaf-html new-tag
					    (string-append "<"
							   (car in-tag)
							   ">"))
		   (container.insert-child new-word 0 new-tag)
		   (container.insert-child parent index new-word)
		   (set! index (+ index 1))
		   (if (eof-object? (cdr in-tag))
		       (set! done #t)))))))


      (close-input-port fp)

      (xword-window.set-doc xw new-doc)

      (xword-window.set-cursor-position
       xw
       (container.first-lowest-container new-doc)
       0)

      (xword-window.rewrap xw
			   (container.first-lowest-container new-doc)
			   (container.last-lowest-container new-doc))

      (xword-window.scroll-to-cursor xw)
      (xword-window.redraw xw new-doc)

      new-doc)))


(define (load-html-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (load-html doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) load-html-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) load-html-fsb-cancel (list #f)))


;; some html goo to look at

;FAQ162.html:</pre>&lt;hr&gt;
;FAQ18.html:<A href="mailto:doc@FreeBSD.ORG">&lt;doc@FreeBSD.ORG&gt;</A>.</p>
;FAQ18.html:<p>Evi Nemeth, Garth Snyder, Scott Seebass &amp; Trent R. Hein,<br>
;FAQ18.html:(approx. US&#36;45-&#36;55), but worth it.  It also
