;;;;;;;;;
;; save
;;;;;;;;;

(define *filename* '())
(define *current-font* '())		; list, car = name, cdr = size

(define (save-ps-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-save #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-save #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-save-ps widget client-data call-data))
		      handler)))

    (cond ((eq? can-save #f)
	   (display "can't save ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline)))))

(define (actually-save-ps widget client-data call-data)
;(define (save-ps-fsb-ok widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "w"))))

    (if (eq? fp #f)
	(begin
	  (display "can't save file.")
	  (newline)
	  (error "can't save")))
    
    (if (eq? *filename* '())
	(set! *filename* file-name))
    (set! *current-font* (list #f #f))

    (save-ps-header fp) ; output the postscript header

    (save-ps-container doc fp 0 0)

    (display "%%EOF" fp) ; end file
    (newline fp)

    (close-output-port fp)))
	


(define (save-ps-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (save-ps doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) save-ps-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) save-ps-fsb-cancel (list #f)))

;;;;

(define (save-ps-header fp)
  (display "%!PS-Adobe-2.0" fp)
  (newline fp)
  (display "%% Creator: XWord Version 0.1" fp)
  (newline fp)
  (display "%% DocumentName: " fp)
  (display *filename* fp)
  (newline fp)
  (display "%% Author: " fp)
  (display (getenv "LOGNAME") fp)
  (newline fp)
  (display "%%EndComments" fp)
  (newline fp)
  (newline fp))


(define (save-ps-container con fp x y)
  (let* ((dir (container.get-attribute con at-direction))
	 (init-and-extra (container.handle-fill con))
	 (init (car init-and-extra))
	 (extra (cadr init-and-extra))
	 (lx x)
	 (ly y))

    (cond ((eqv? dir DIR-rl) (set! lx (- lx init)))
	  ((eqv? dir DIR-du) (set! ly (- ly init)))
	  ((eqv? dir DIR-lr) (set! lx (+ lx init)))
	  ((eqv? dir DIR-ud) (set! ly (+ ly init))))

    (do ((child-index 0 (+ child-index 1)))
	((>= child-index (container.get-num-children con)) #t)

      (let* ((child (container.get-child con child-index))
	     (child-type (container.get-type child))
	     (child-size (container.get-size child))
	     (left (/ (car child-size) magnify))
	     (up (/ (cadr child-size) magnify))
	     (right (/ (caddr child-size) magnify))
	     (down (/ (cadddr child-size) magnify))
	     )
	(cond ((eqv? dir DIR-rl) (set! lx (- lx right)))
	      ((eqv? dir DIR-du) (set! ly (- ly down)))
	      ((eqv? dir DIR-lr) (set! lx (+ lx left)))
	      ((eqv? dir DIR-ud) (set! ly (+ ly up))))

	(cond ((string=? child-type "WordSegment")
	       (save-ps-wordsegment child fp lx ly))
	      ((string=? child-type "Tag")
	       (save-ps-tag child fp lx ly))
	      ((string=? child-type "Page")
	       (save-ps-page child fp lx ly))
	      (else
	       (save-ps-container child fp lx ly)))

	(cond ((eqv? dir DIR-rl) (set! lx (- lx left extra)))
	      ((eqv? dir DIR-du) (set! ly (- ly up extra)))
	      ((eqv? dir DIR-lr) (set! lx (+ lx right extra)))
	      ((eqv? dir DIR-ud) (set! ly (+ ly down extra))))
	) ; end of let*
      ) ; end of do
    ) ; end of let*
  ) ; end of define



(define (save-ps-page page fp x y)
  ;(save-ps-container page fp x y)
  (save-ps-container page fp x 0)
  (display "showpage" fp) (newline fp))


(define (save-ps-wordsegment ws fp lx ly)
  (let* ((dir (container.get-attribute ws at-direction))
	 (str (container.get-leaf-ps xd ws))
	 (font (container.get-attribute ws at-font))
	 (font-name (vk-font.display-name font))
	 (font-size (vk-font.point-size font))

	 (child-size (container.get-size ws))
	 (left (/ (car child-size) magnify))
	 (right (/ (caddr child-size) magnify))
	 )

    (if (eqv? dir DIR-rl)
	(set! lx (- lx left right)))


    ; remove trailing whitespace (and returns)
    (let ((str-len (string-length str)))
      (if (> str-len 0)
	  (let ((last-char (string-ref str (- str-len 1))))
	    (if (or (char=? last-char #\Space)
		    (char=? last-char #\Newline)
		    (char=? last-char #\015))
		(set! str (substring str 0 (- str-len 1)))))))

    ; bail out if this word is just whitespace.
    (cond ((> (string-length str) 0)

	   ; replace ( with \( and ) with \)
	   (do ((i 0 (+ i 1))
		(str-len (length str))
		(c #f)
		)
	       ((>= i str-len) #t)
	     (set! c (string-ref str i))
	     (if (char=? c #\()
		 (begin
		   (set! str (string-append (substring str 0 i)
					    "\\("
					    (substring str (+ i 1) str-len)))
		   (set! i (+ i 1))
		   (set! str-len (+ str-len 1))))
	     (if (char=? c #\))
		 (begin
		   (set! str (string-append (substring str 0 i)
					    "\\)"
					    (substring str (+ i 1) str-len)))
		   (set! i (+ i 1))
		   (set! str-len (+ str-len 1)))))

	   ; set position
	   (display (+ 72 lx) fp)
	   (display " " fp)
	   (display (- (* 72 10) ly) fp)
	   (display " moveto " fp)

	   ; keep track of current font.  don't re-set the same font
	   (cond  ((not (equal? (list font-name font-size) *current-font*))
		   (display "/" fp)
		   (display font-name fp)
		   (display " findfont " fp)
		   (display font-size fp)
		   (display " scalefont setfont" fp)
		   (newline fp)
		   (set! *current-font* (list font-name font-size))))

	   ; display str
	   (display "(" fp)
	   (display str fp)
	   (display ")" fp)
	   (display " show" fp)
	   (newline fp)))))


(define (save-ps-tag tag fp lx ly)
  (let* ((str (container.get-leaf-ps xd tag))
	 (type (substring str 0 6))
	 )
    (cond ((string=? type " skip:") #t)
	  ;
	  ((string=? type "shell:")
	   (display "gsave" fp) (newline fp)
	   (display (+ 72 lx) fp)
	   (display " " fp)
	   (display (- (* 72 10) ly) fp)
	   (display " translate " fp) (newline fp)

	   ; run the shell command
	   (let* ((p (pipe))
		  (stdout (car (fdes->ports 1)))
		  )
	     (if (eqv? 0 (primitive-fork))
		 (begin
		   (close-port (car p))
		   (redirect-port (cdr p) stdout)
		   (execlp-bleh (substring str 6 (length str)))
		   ))
	     (close-port (cdr p))

	     ; append whatever comes out of p onto fp
	     (do ((line (read-line (car p))
			(read-line (car p))))
		 ((eof-object? line) #t)
	       ; kill comments... they will confuse the interpreter.
	       (if (or (< (length line) 1)
		       (not (string-ci=? "%" (substring line 0 1))))
		   ; try to kill showpages
		   (if (or (< (length line) 8)
			   (not (string-ci=? "showpage"
					     (substring line 0 8))))
		       (begin
			 (display line fp)
			 (newline fp))))
	       ) ; of do
	     (close-port (car p))
	     ) ; of let*
	   (display "grestore" fp) (newline fp)
	   ) ; of string=? shell:
	  ;
	  ((string=? type "   ps:")
	   (display (+ 72 lx) fp)
	   (display " " fp)
	   (display (- (* 72 10) ly) fp)
	   (display " moveto " fp)
	   (display (substring str 6 (length str)) fp))
	  ;
	  (else #t)
	  )
    ))

