;;;;;;;;;
;; save
;;;;;;;;;

(define (save-xword-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-save #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-save #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-save-xword widget client-data call-data))
		      handler)))

    (cond ((eq? can-save #f)
	   (display "can't save ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline)))))

(define (actually-save-xword widget client-data call-data)

  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "w"))))

    (if (eq? fp #f)
	(begin
	  (display "can't save file.")
	  (newline)
	  (error "can't save")))

    (save-xword-header fp) ; output the header
    (save-xword-attributes fp)
    (save-xword-container doc fp 0 0)
    (close-output-port fp)))



(define (save-xword-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (save-xword doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) save-xword-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) save-xword-fsb-cancel (list #f)))

;;;;

(define (save-xword-header fp)
  #t)


(define (save-xword-attributes fp)
  (let* ((len (length local-wordsegment-styles)))
    (do ((i 0 (+ i 1)))
	((>= i len) #t)
      (let* ((nth (list-ref local-wordsegment-styles i))
	     (font (attributes.get-i nth at-font))
	     (name (vk-font.display-name font))
	     (size (vk-font.point-size font)))
	(display "A:0:0:" fp)
	(display name fp)
	(display "." fp)
	(display size fp)
	(newline fp)))))


(define (save-xword-container con fp x y)
  (let* ((attrs (container.get-attributes con))
	 (attrs-num (list-nth attrs all-styles))

	 (size (container.get-size con))
	 (left (car size))
	 (up (cadr size))
	 (right (caddr size))
	 (down (cadddr size))

	 (num-ch (container.get-num-children con))

	 (type (container.get-type con))
	 )


    (cond ((string=? type "Document")
	   (display "D:" fp))
	  ((string=? type "Page")
	   (display "P:" fp))
	  ((string=? type "Container")
	   (display "C:" fp))
	  ;((string=? type "WordSegment")
	   ;(display "W:" fp))
	  (else
	   (display "can't save '")
	   (display type)
	   (display "'")
	   (newline)))
	   

    (display attrs-num fp) (display ":" fp)
    ;(display left fp) (display ":" fp)
    ;(display up fp) (display ":" fp)
    ;(display right fp) (display ":" fp)
    ;(display down fp) (display ":" fp)
    (display num-ch fp) (display ":" fp)
    (newline fp)


    (do ((child-index 0 (+ child-index 1)))
	((>= child-index (container.get-num-children con)) #t)

      (let* ((child (container.get-child con child-index))
	     (child-type (container.get-type child))
	     )
	(cond ((string=? child-type "WordSegment")
	       (save-xword-wordsegment child fp x y))
	      ((string=? child-type "Tag")
	       (save-xword-tag child fp x y))
	      (else
	       (save-xword-container child fp x y)))
	) ; end of let*
      ) ; end of do
    ) ; end of let*
  ) ; end of define



(define (save-xword-wordsegment ws fp lx ly)
  (let* (
	 (attrs (container.get-attributes ws))
	 (attrs-num (list-nth attrs all-styles))
	 (num-ch (container.get-num-children ws))
	 (str (container.get-leaf-xword ws))
	 )

    (display "W:" fp)
    (display attrs-num fp) (display ":" fp)
    (display num-ch fp) (display ":" fp)
    (display str fp)
    (newline fp)))


(define (save-xword-tag tg fp lx ly)
  (let* (
	 (attrs (container.get-attributes tg))
	 (attrs-num (list-nth attrs all-styles))
	 (num-ch (container.get-num-children tg))
	 (str (container.get-leaf-xword tg))
	 )

    (display "T:" fp)
    (display attrs-num fp) (display ":" fp)
    (display num-ch fp) (display ":" fp)
    (display str fp)
    (newline fp)))


;;;;;;;;;
;; load
;;;;;;;;;


(define (load-xword-fsb-cancel widget client-data call-data)
  (xt-popdown fsb-shell))


(define (load-xword doc)
  (show-fsb)
  (xt-remove-all-callbacks fsb (xm-n-ok-callback))
  (xt-remove-all-callbacks fsb (xm-n-cancel-callback))
  (xt-add-callback fsb (xm-n-ok-callback) load-xword-fsb-ok (list #f))
  (xt-add-callback fsb (xm-n-cancel-callback) load-xword-fsb-cancel (list #f)))


;(define (handler key args . rest)
;  (set! can-load #f)
;  key)


(define (load-xword-fsb-ok widget client-data call-data)
  (xt-popdown fsb-shell)

  (let* ((can-load #t)
	 (handler
	  (lambda (key args . rest)
	    (set! can-load #f)
	    key))
	 (blah (catch #t
		      (lambda ()
			(actually-load-xword widget client-data call-data))
		      handler)))

    (cond ((eq? can-load #f)
	   (display "can't load ")
	   (display (fsb-get-file-name widget))
	   (newline)
	   (display "exception: ")
	   (display blah)
	   (newline))
	  (else
	   (set! doc blah)
	   (xword-window.set-doc xw doc)
	   (xword-window.set-cursor-position
	    xw
	    (container.first-lowest-container doc)
	    0)
	   
	   (xword-window.rewrap xw
				(container.first-lowest-container doc)
				(container.last-lowest-container doc))

	   (xword-window.scroll-to-cursor xw)
	   (xword-window.redraw xw doc)))))


(define (actually-load-xword widget client-data call-data)
  ;; free old document here...
  (let* ((file-name (fsb-get-file-name widget))
	 (fp (false-if-exception
	      (open-file file-name "r")))
	 (new-doc (false-if-exception
		   (load-xword-decode-slab fp))))
    
    (if (eq? fp #f)
	(begin
	  (display "no such file")
	  (newline)
	  (error "no such file")))

    (if (eq? new-doc #f)
	(begin
	  (display "error in loading file.")
	  (newline)
	  (error "load error")))

    (close-input-port fp)
    new-doc))


(define (load-xword-attribute input-line fp)
  (let* ((name-start 6)
	 (name-end (char-index input-line name-start #\.))
	 (size-start (+ name-end 1))
	 (size-end (string-length input-line))
	 (name (substring input-line name-start name-end))
	 (size (string->number (substring input-line size-start size-end))))
    (find-wordsegment-style name size))
  (load-xword-decode-slab fp))


(define (load-xword-decode-slab fp)
  (let* ((input-line (read-line fp 'trim))
	 (type (string-ref input-line 0))
	 (attr-start 2)
	 (attr-end (char-index input-line attr-start #\:))
	 (attr-num (string->number
		    (substring input-line attr-start attr-end)))
	 (attrs (list-ref all-styles attr-num))
	 (num-ch-start (+ attr-end 1))
	 (num-ch-end (char-index input-line num-ch-start #\:))
	 (num-ch (string->number
		  (substring input-line num-ch-start num-ch-end)))
	 (con (cond ((char=? type #\D) (document.new attrs))
		    ((char=? type #\P) (page.new attrs))
		    ((char=? type #\W) (wordsegment.new attrs))
		    ((char=? type #\T) (tag.new attrs))
		    ((char=? type #\C) (container.new attrs))
		    ((char=? type #\A) (load-xword-attribute input-line fp))
		    (else (display "no match") (newline))
		    ))
	 (leaf-txt (substring input-line
			      (+ num-ch-end 1)
			      (length input-line)))
	 )

    ;(display "input-line: ") (display input-line) (newline)
    ;(display "attr-num: ") (display attr-num) (display ", ")
    ;(display "num-ch: ") (display num-ch) (display ", ")
    ;(display "leaf-txt: ") (display leaf-txt) (newline)

    (if (or (char=? type #\W)
	    (char=? type #\T))
	(container.set-leaf-xword con leaf-txt)
	; else not a leaf
	(do ((i 0 (+ i 1)))
	    ((>= i num-ch) con)
	  (container.insert-child con
				  (container.get-num-children con)
				  (load-xword-decode-slab fp))))
    con))
