 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; code to tell which modifier keys are being pressed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define key-state-shift 1)
(define key-state-shift-lock 2)
(define key-state-control 4)
(define key-state-alt 8)
(define key-state-meta 16)
(define Mod3Mask 32)
(define Mod4Mask 64)
(define Mod5Mask 128)
(define Button1Mask 256)
(define Button2Mask 512)
(define Button3Mask 1024)
(define Button4Mask 2048)
(define Button5Mask 4096)


;; bitwise operations??? help!  I hate to require slib just for this...
;; FIX ME!!!

(define (key-state-meta? st)
  (>= st key-state-meta))

(define (key-state-alt? st)
  (if (key-state-meta? st) (set! st (- st key-state-meta)))
  (>= st key-state-alt))

(define (key-state-control? st)
  (if (key-state-meta? st) (set! st (- st key-state-meta)))
  (if (key-state-alt? st) (set! st (- st key-state-meta)))
  (>= st key-state-control))

(define (key-state-shift-lock? st)
  (if (key-state-meta? st) (set! st (- st key-state-meta)))
  (if (key-state-alt? st) (set! st (- st key-state-alt)))
  (if (key-state-control? st) (set! st (- st key-state-control)))
  (>= st key-state-shift-lock))

(define (key-state-shift? st)
  (if (key-state-meta? st) (set! st (- st key-state-meta)))
  (if (key-state-alt? st) (set! st (- st key-state-alt)))
  (if (key-state-control? st) (set! st (- st key-state-control)))
  (if (key-state-shift-lock? st) (set! st (- st key-state-shift-lock)))
  (>= st key-state-shift))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (prev-ws con)
  (do ((c (container.prev-node con) (container.prev-node c)))
      ((or (eqv? c #f)
	   (string=? (container.get-type c) "WordSegment"))
       c)))

(define (next-ws con)
  (do ((c (container.next-node con) (container.next-node c)))
      ((or (eqv? c #f)
	   (string=? (container.get-type c) "WordSegment"))
       c)))


(define (move-cursor-backward)
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (con (car cursor))
	 (offset (cadr cursor))
	 )
    (if (> offset 0)
	(set! offset (- offset 1))
	;else offset == 0
	(let* ((prev (prev-ws con)))
	  (if prev
	      (begin (set! con prev)
		     (set! offset (container.get-num-children con))
		     (if (> offset 0)
			 (set! offset (- offset 1)))))))
    (xword-window.set-cursor-position xw con offset))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (move-cursor-forward)
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (con (car cursor))
	 (offset (cadr cursor))
	 )
    (if (< offset (container.get-num-children con))
	(set! offset (+ offset 1))
	;else offset >= num_ch
	(let* ((next (next-ws con)))
	  (if next
	      (begin (set! con next)
		     (set! offset 0)
		     (if (< offset (container.get-num-children con))
			 (set! offset (+ offset 1)))))))
    (xword-window.set-cursor-position xw con offset))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (move-cursor-up)
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (wordsegment (car cursor))
	 (word (container.get-parent wordsegment))
	 (line (container.get-parent word))
	 (line-dir (container.get-attribute line at-direction))
	 (line-size (if (or (eqv? line-dir DIR-lr)
			    (eqv? line-dir DIR-rl))
			(container.get-size line)
			(container.get-size word)))
	 (up (cadr line-size))
	 (xy (xword-window.get-cursor-xy xw))
	 (x (car xy))
	 (y (- (cadr xy) up 2)))
    (xword-window.set-cursor-xy xw x y))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (move-cursor-down)
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (wordsegment (car cursor))
	 (word (container.get-parent wordsegment))
	 (line (container.get-parent word))
	 (line-dir (container.get-attribute line at-direction))
	 (line-size (if (or (eqv? line-dir DIR-lr)
			    (eqv? line-dir DIR-rl))
			(container.get-size line)
			(container.get-size word)))
	 (down (cadddr line-size))
	 (xy (xword-window.get-cursor-xy xw))
	 (x (car xy))
	 (y (+ (cadr xy) down 2)))
    (xword-window.set-cursor-xy xw x y))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (move-cursor-sol) ; start of line
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (wordsegment (car cursor))
	 (word (container.get-parent wordsegment))
	 (line (container.get-parent word))
	 (con (container.first-lowest-container line))
	 (offset 0))
    (xword-window.set-cursor-position xw con offset))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (move-cursor-eol) ; start of line
  (let* ((cursor (xword-window.get-cursor-position xw))
	 (wordsegment (car cursor))
	 (word (container.get-parent wordsegment))
	 (line (container.get-parent word))
	 (con (container.last-lowest-container line))
	 (offset (container.get-num-children con)))
    (xword-window.set-cursor-position xw con offset))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; define reactions to various keys being pressed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (do-key-backspace state k)
  (if (eqv? (clear-selected-area) #f)
      (let ((top (xword-window.delete-character xw)))
	(xword-window.scroll-to-cursor xw)
	(xword-window.redraw xw top))))


(define (do-key-left state k)
  (if (eqv? (container.get-attribute
	     (car (xword-window.get-cursor-position xw))
	     at-direction)
	    DIR-lr)
  (move-cursor-backward)
  (move-cursor-forward))
  )

(define (do-key-up state k)
  (move-cursor-up))

(define (do-key-right state k)
  (if (eqv? (container.get-attribute
	     (car (xword-window.get-cursor-position xw))
	     at-direction)
	    DIR-lr)
      (move-cursor-forward)
      (move-cursor-backward)))

(define (do-key-down state k)
  (move-cursor-down))

(define (do-key-start-of-line state k)
  (if (eqv? (container.get-attribute
	     (car (xword-window.get-cursor-position xw))
	     at-direction)
	    DIR-lr)
  (move-cursor-sol)
  (move-cursor-eol)))

(define (do-key-end-of-line state k)
  (if (eqv? (container.get-attribute
	     (car (xword-window.get-cursor-position xw))
	     at-direction)
	    DIR-lr)
  (move-cursor-eol)
  (move-cursor-sol)))

(define (do-set-mark state k)
  ;(display "set-mark-command") (newline)
  #t)


(define (do-key-ignore state k) #t)


(define (do-key-insert state k)
  (clear-selected-area)
  (let* ((top (xword-window.insert-xk xw k cur-attr)))
    (xword-window.scroll-to-cursor xw)
    (xword-window.redraw xw top)))


(define (do-key-insert-tag state k)
  (clear-selected-area)
  (let* ((tag-text (cond ((eqv? k XK_Return) "<br>")
			 ((eqv? k XK_F1) "<hr>")))
	 (top (xword-window.insert-tag xw tag-text cur-tag-attr)))
    (xword-window.scroll-to-cursor xw)
    (xword-window.redraw xw top)))


(define (do-key-copy state k)
  (copy-sel))

(define (do-key-paste state k)
  (paste-sel)
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))

(define (do-key-cut state k)
  (cut-sel)
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; look up which proc to run given a key
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define key-procs
  (list (list XK_BackSpace  do-key-backspace)
	(list XK_Delete     do-key-backspace)
	(list XK_Left       do-key-left)
	(list XK_Right      do-key-right)
	(list XK_Up         do-key-up)
	(list XK_Down       do-key-down)
	(list XK_Home       do-key-start-of-line)
	(list XK_End        do-key-end-of-line)
	(list XK_Shift_L    do-key-ignore)
	(list XK_Shift_R    do-key-ignore)
	(list XK_Control_L  do-key-ignore)
	(list XK_Control_R  do-key-ignore)
	(list XK_Caps_Lock  do-key-ignore)
	(list XK_Shift_Lock do-key-ignore)
	(list XK_Alt_L      do-key-ignore)
	(list XK_Alt_R      do-key-ignore)
	(list XK_Meta_L     do-key-ignore)
	(list XK_Meta_R     do-key-ignore)
	(list XK_Prior      do-key-ignore)
	(list XK_Next       do-key-ignore)
	(list XK_Begin      do-key-ignore)
	(list XK_Multi_key  do-key-ignore)
	(list XK_F1         do-key-insert-tag)
	(list XK_Return     do-key-insert-tag)
	(list 0             do-key-insert)))

(define key-procs-ctrl
  (list (list XK_a          do-key-start-of-line)
	(list XK_A          do-key-start-of-line)
	(list XK_b          do-key-left)
	(list XK_B          do-key-left)
	(list XK_e          do-key-end-of-line)
	(list XK_E          do-key-end-of-line)
	(list XK_f          do-key-right)
	(list XK_F          do-key-right)
	(list XK_p          do-key-up)
	(list XK_P          do-key-up)
	(list XK_n          do-key-down)
	(list XK_N          do-key-down)
	(list XK_space      do-set-mark)
	(list 0             do-key-ignore)))


(define key-procs-meta
  (list (list XK_c          do-key-copy)
	(list XK_C          do-key-copy)
	(list XK_v          do-key-paste)
	(list XK_V          do-key-paste)
	(list XK_x          do-key-cut)
	(list XK_X          do-key-cut)
	(list 0             do-key-ignore)))

(define key-procs-shift-meta
  (list (list 0             do-key-ignore)))


;;;;;;;;

(define (key-state->table st)
  (if (key-state-control? st)
      key-procs-ctrl
      (if (and (key-state-shift? st)
	       (or (key-state-meta? st)
		   (key-state-alt? st)))
	  key-procs-shift-meta
	  (if (or (key-state-meta? st)
		  (key-state-alt? st))
	      key-procs-meta
	      key-procs))))
