; TTY Functions, cleverly chosen to map to ascii, for convenience of
; programming, but could have been arbitrary (at the cost of lookup
; tables in client code.


(define XK_BackSpace		#xFF08)  ; back space, back char
(define XK_Tab			#xFF09)
(define XK_Linefeed		#xFF0A)  ; Linefeed, LF
(define XK_Clear		#xFF0B)
(define XK_Return		#xFF0D)  ; Return, enter
(define XK_Pause		#xFF13)  ; Pause, hold
(define XK_Scroll_Lock		#xFF14)
(define XK_Sys_Req		#xFF15)
(define XK_Escape		#xFF1B)
(define XK_Delete		#xFFFF)  ; Delete, rubout


; International & multi-key character composition

(define XK_Multi_key		#xFF20)  ; Multi-key character compose

; Japanese keyboard support

(define XK_Kanji		#xFF21)  ; Kanji, Kanji convert
(define XK_Muhenkan		#xFF22)  ; Cancel Conversion
(define XK_Henkan_Mode		#xFF23)  ; Start/Stop Conversion
(define XK_Henkan		#xFF23)  ; Alias for Henkan_Mode
(define XK_Romaji		#xFF24)  ; to Romaji
(define XK_Hiragana		#xFF25)  ; to Hiragana
(define XK_Katakana		#xFF26)  ; to Katakana
(define XK_Hiragana_Katakana	#xFF27)  ; Hiragana/Katakana toggle
(define XK_Zenkaku		#xFF28)  ; to Zenkaku
(define XK_Hankaku		#xFF29)  ; to Hankaku
(define XK_Zenkaku_Hankaku	#xFF2A)  ; Zenkaku/Hankaku toggle
(define XK_Touroku		#xFF2B)  ; Add to Dictionary
(define XK_Massyo		#xFF2C)  ; Delete from Dictionary
(define XK_Kana_Lock		#xFF2D)  ; Kana Lock
(define XK_Kana_Shift		#xFF2E)  ; Kana Shift
(define XK_Eisu_Shift		#xFF2F)  ; Alphanumeric Shift
(define XK_Eisu_toggle		#xFF30)  ; Alphanumeric toggle

; #xFF31 thru #xFF3F are under XK_KOREAN

; Cursor control & motion

(define XK_Home			#xFF50)
(define XK_Left			#xFF51)  ; Move left, left arrow
(define XK_Up			#xFF52)  ; Move up, up arrow
(define XK_Right		#xFF53)  ; Move right, right arrow
(define XK_Down			#xFF54)  ; Move down, down arrow
(define XK_Prior		#xFF55)  ; Prior, previous
(define XK_Page_Up		#xFF55)
(define XK_Next			#xFF56)  ; Next
(define XK_Page_Down		#xFF56)
(define XK_End			#xFF57)  ; EOL
(define XK_Begin		#xFF58)  ; BOL


; Misc Functions

(define XK_Select		#xFF60)  ; Select, mark
(define XK_Print		#xFF61)
(define XK_Execute		#xFF62)  ; Execute, run, do
(define XK_Insert		#xFF63)  ; Insert, insert here
(define XK_Undo			#xFF65)  ; Undo, oops
(define XK_Redo			#xFF66)  ; redo, again
(define XK_Menu			#xFF67)
(define XK_Find			#xFF68)  ; Find, search
(define XK_Cancel		#xFF69)  ; Cancel, stop, abort, exit
(define XK_Help			#xFF6A)  ; Help
(define XK_Break		#xFF6B)
(define XK_Mode_switch		#xFF7E)  ; Character set switch
(define XK_script_switch        #xFF7E)  ; Alias for mode_switch
(define XK_Num_Lock		#xFF7F)

; Keypad Functions, keypad numbers cleverly chosen to map to ascii

(define XK_KP_Space		#xFF80)  ; space
(define XK_KP_Tab		#xFF89)
(define XK_KP_Enter		#xFF8D)  ; enter
(define XK_KP_F1		#xFF91)  ; PF1, KP_A, ...
(define XK_KP_F2		#xFF92)
(define XK_KP_F3		#xFF93)
(define XK_KP_F4		#xFF94)
(define XK_KP_Home		#xFF95)
(define XK_KP_Left		#xFF96)
(define XK_KP_Up		#xFF97)
(define XK_KP_Right		#xFF98)
(define XK_KP_Down		#xFF99)
(define XK_KP_Prior		#xFF9A)
(define XK_KP_Page_Up		#xFF9A)
(define XK_KP_Next		#xFF9B)
(define XK_KP_Page_Down		#xFF9B)
(define XK_KP_End		#xFF9C)
(define XK_KP_Begin		#xFF9D)
(define XK_KP_Insert		#xFF9E)
(define XK_KP_Delete		#xFF9F)
(define XK_KP_Equal		#xFFBD)  ; equals
(define XK_KP_Multiply		#xFFAA)
(define XK_KP_Add		#xFFAB)
(define XK_KP_Separator		#xFFAC)  ; separator, often comma
(define XK_KP_Subtract		#xFFAD)
(define XK_KP_Decimal		#xFFAE)
(define XK_KP_Divide		#xFFAF)

(define XK_KP_0			#xFFB0)
(define XK_KP_1			#xFFB1)
(define XK_KP_2			#xFFB2)
(define XK_KP_3			#xFFB3)
(define XK_KP_4			#xFFB4)
(define XK_KP_5			#xFFB5)
(define XK_KP_6			#xFFB6)
(define XK_KP_7			#xFFB7)
(define XK_KP_8			#xFFB8)
(define XK_KP_9			#xFFB9)



; Auxilliary Functions; note the duplicate definitions for left and right
; function keys;  Sun keyboards and a few other manufactures have such
; function key groups on the left and/or right sides of the keyboard.
; We've not found a keyboard with more than 35 function keys total.


(define XK_F1			#xFFBE)
(define XK_F2			#xFFBF)
(define XK_F3			#xFFC0)
(define XK_F4			#xFFC1)
(define XK_F5			#xFFC2)
(define XK_F6			#xFFC3)
(define XK_F7			#xFFC4)
(define XK_F8			#xFFC5)
(define XK_F9			#xFFC6)
(define XK_F10			#xFFC7)
(define XK_F11			#xFFC8)
(define XK_L1			#xFFC8)
(define XK_F12			#xFFC9)
(define XK_L2			#xFFC9)
(define XK_F13			#xFFCA)
(define XK_L3			#xFFCA)
(define XK_F14			#xFFCB)
(define XK_L4			#xFFCB)
(define XK_F15			#xFFCC)
(define XK_L5			#xFFCC)
(define XK_F16			#xFFCD)
(define XK_L6			#xFFCD)
(define XK_F17			#xFFCE)
(define XK_L7			#xFFCE)
(define XK_F18			#xFFCF)
(define XK_L8			#xFFCF)
(define XK_F19			#xFFD0)
(define XK_L9			#xFFD0)
(define XK_F20			#xFFD1)
(define XK_L10			#xFFD1)
(define XK_F21			#xFFD2)
(define XK_R1			#xFFD2)
(define XK_F22			#xFFD3)
(define XK_R2			#xFFD3)
(define XK_F23			#xFFD4)
(define XK_R3			#xFFD4)
(define XK_F24			#xFFD5)
(define XK_R4			#xFFD5)
(define XK_F25			#xFFD6)
(define XK_R5			#xFFD6)
(define XK_F26			#xFFD7)
(define XK_R6			#xFFD7)
(define XK_F27			#xFFD8)
(define XK_R7			#xFFD8)
(define XK_F28			#xFFD9)
(define XK_R8			#xFFD9)
(define XK_F29			#xFFDA)
(define XK_R9			#xFFDA)
(define XK_F30			#xFFDB)
(define XK_R10			#xFFDB)
(define XK_F31			#xFFDC)
(define XK_R11			#xFFDC)
(define XK_F32			#xFFDD)
(define XK_R12			#xFFDD)
(define XK_F33			#xFFDE)
(define XK_R13			#xFFDE)
(define XK_F34			#xFFDF)
(define XK_R14			#xFFDF)
(define XK_F35			#xFFE0)
(define XK_R15			#xFFE0)

; Modifiers

(define XK_Shift_L		#xFFE1)  ; Left shift
(define XK_Shift_R		#xFFE2)  ; Right shift
(define XK_Control_L		#xFFE3)  ; Left control
(define XK_Control_R		#xFFE4)  ; Right control
(define XK_Caps_Lock		#xFFE5)  ; Caps lock
(define XK_Shift_Lock		#xFFE6)  ; Shift lock

(define XK_Meta_L		#xFFE7)  ; Left meta
(define XK_Meta_R		#xFFE8)  ; Right meta
(define XK_Alt_L		#xFFE9)  ; Left alt
(define XK_Alt_R		#xFFEA)  ; Right alt
(define XK_Super_L		#xFFEB)  ; Left super
(define XK_Super_R		#xFFEC)  ; Right super
(define XK_Hyper_L		#xFFED)  ; Left hyper
(define XK_Hyper_R		#xFFEE)  ; Right hyper


; ISO 9995 Function and Modifier Keys
; Byte 3 = #xFE


(define	XK_ISO_Lock					#xFE01)
(define	XK_ISO_Level2_Latch				#xFE02)
(define	XK_ISO_Level3_Shift				#xFE03)
(define	XK_ISO_Level3_Latch				#xFE04)
(define	XK_ISO_Level3_Lock				#xFE05)
(define	XK_ISO_Group_Shift		#xFF7E)  ; Alias for mode_switch
(define	XK_ISO_Group_Latch				#xFE06)
(define	XK_ISO_Group_Lock				#xFE07)
(define	XK_ISO_Next_Group				#xFE08)
(define	XK_ISO_Next_Group_Lock				#xFE09)
(define	XK_ISO_Prev_Group				#xFE0A)
(define	XK_ISO_Prev_Group_Lock				#xFE0B)
(define	XK_ISO_First_Group				#xFE0C)
(define	XK_ISO_First_Group_Lock				#xFE0D)
(define	XK_ISO_Last_Group				#xFE0E)
(define	XK_ISO_Last_Group_Lock				#xFE0F)

(define	XK_ISO_Left_Tab					#xFE20)
(define	XK_ISO_Move_Line_Up				#xFE21)
(define	XK_ISO_Move_Line_Down				#xFE22)
(define	XK_ISO_Partial_Line_Up				#xFE23)
(define	XK_ISO_Partial_Line_Down			#xFE24)
(define	XK_ISO_Partial_Space_Left			#xFE25)
(define	XK_ISO_Partial_Space_Right			#xFE26)
(define	XK_ISO_Set_Margin_Left				#xFE27)
(define	XK_ISO_Set_Margin_Right				#xFE28)
(define	XK_ISO_Release_Margin_Left			#xFE29)
(define	XK_ISO_Release_Margin_Right			#xFE2A)
(define	XK_ISO_Release_Both_Margins			#xFE2B)
(define	XK_ISO_Fast_Cursor_Left				#xFE2C)
(define	XK_ISO_Fast_Cursor_Right			#xFE2D)
(define	XK_ISO_Fast_Cursor_Up				#xFE2E)
(define	XK_ISO_Fast_Cursor_Down				#xFE2F)
(define	XK_ISO_Continuous_Underline			#xFE30)
(define	XK_ISO_Discontinuous_Underline			#xFE31)
(define	XK_ISO_Emphasize				#xFE32)
(define	XK_ISO_Center_Object				#xFE33)
(define	XK_ISO_Enter					#xFE34)

(define	XK_dead_grave					#xFE50)
(define	XK_dead_acute					#xFE51)
(define	XK_dead_circumflex				#xFE52)
(define	XK_dead_tilde					#xFE53)
(define	XK_dead_macron					#xFE54)
(define	XK_dead_breve					#xFE55)
(define	XK_dead_abovedot				#xFE56)
(define	XK_dead_diaeresis				#xFE57)
(define	XK_dead_abovering				#xFE58)
(define	XK_dead_doubleacute				#xFE59)
(define	XK_dead_caron					#xFE5A)
(define	XK_dead_cedilla					#xFE5B)
(define	XK_dead_ogonek					#xFE5C)
(define	XK_dead_iota					#xFE5D)
(define	XK_dead_voiced_sound				#xFE5E)
(define	XK_dead_semivoiced_sound			#xFE5F)
(define	XK_dead_belowdot				#xFE60)

(define	XK_First_Virtual_Screen				#xFED0)
(define	XK_Prev_Virtual_Screen				#xFED1)
(define	XK_Next_Virtual_Screen				#xFED2)
(define	XK_Last_Virtual_Screen				#xFED4)
(define	XK_Terminate_Server				#xFED5)

(define	XK_AccessX_Enable				#xFE70)
(define	XK_AccessX_Feedback_Enable			#xFE71)
(define	XK_RepeatKeys_Enable				#xFE72)
(define	XK_SlowKeys_Enable				#xFE73)
(define	XK_BounceKeys_Enable				#xFE74)
(define	XK_StickyKeys_Enable				#xFE75)
(define	XK_MouseKeys_Enable				#xFE76)
(define	XK_MouseKeys_Accel_Enable			#xFE77)
(define	XK_Overlay1_Enable				#xFE78)
(define	XK_Overlay2_Enable				#xFE79)
(define	XK_AudibleBell_Enable				#xFE7A)

(define	XK_Pointer_Left					#xFEE0)
(define	XK_Pointer_Right				#xFEE1)
(define	XK_Pointer_Up					#xFEE2)
(define	XK_Pointer_Down					#xFEE3)
(define	XK_Pointer_UpLeft				#xFEE4)
(define	XK_Pointer_UpRight				#xFEE5)
(define	XK_Pointer_DownLeft				#xFEE6)
(define	XK_Pointer_DownRight				#xFEE7)
(define	XK_Pointer_Button_Dflt				#xFEE8)
(define	XK_Pointer_Button1				#xFEE9)
(define	XK_Pointer_Button2				#xFEEA)
(define	XK_Pointer_Button3				#xFEEB)
(define	XK_Pointer_Button4				#xFEEC)
(define	XK_Pointer_Button5				#xFEED)
(define	XK_Pointer_DblClick_Dflt			#xFEEE)
(define	XK_Pointer_DblClick1				#xFEEF)
(define	XK_Pointer_DblClick2				#xFEF0)
(define	XK_Pointer_DblClick3				#xFEF1)
(define	XK_Pointer_DblClick4				#xFEF2)
(define	XK_Pointer_DblClick5				#xFEF3)
(define	XK_Pointer_Drag_Dflt				#xFEF4)
(define	XK_Pointer_Drag1				#xFEF5)
(define	XK_Pointer_Drag2				#xFEF6)
(define	XK_Pointer_Drag3				#xFEF7)
(define	XK_Pointer_Drag4				#xFEF8)
(define	XK_Pointer_Drag5				#xFEFD)

(define	XK_Pointer_EnableKeys				#xFEF9)
(define	XK_Pointer_Accelerate				#xFEFA)
(define	XK_Pointer_DfltBtnNext				#xFEFB)
(define	XK_Pointer_DfltBtnPrev				#xFEFC)


;
; 3270 Terminal Keys
; Byte 3 = #xFD


(define XK_3270_Duplicate      #xFD01)
(define XK_3270_FieldMark      #xFD02)
(define XK_3270_Right2         #xFD03)
(define XK_3270_Left2          #xFD04)
(define XK_3270_BackTab        #xFD05)
(define XK_3270_EraseEOF       #xFD06)
(define XK_3270_EraseInput     #xFD07)
(define XK_3270_Reset          #xFD08)
(define XK_3270_Quit           #xFD09)
(define XK_3270_PA1            #xFD0A)
(define XK_3270_PA2            #xFD0B)
(define XK_3270_PA3            #xFD0C)
(define XK_3270_Test           #xFD0D)
(define XK_3270_Attn           #xFD0E)
(define XK_3270_CursorBlink    #xFD0F)
(define XK_3270_AltCursor      #xFD10)
(define XK_3270_KeyClick       #xFD11)
(define XK_3270_Jump           #xFD12)
(define XK_3270_Ident          #xFD13)
(define XK_3270_Rule           #xFD14)
(define XK_3270_Copy           #xFD15)
(define XK_3270_Play           #xFD16)
(define XK_3270_Setup          #xFD17)
(define XK_3270_Record         #xFD18)
(define XK_3270_ChangeScreen   #xFD19)
(define XK_3270_DeleteWord     #xFD1A)
(define XK_3270_ExSelect       #xFD1B)
(define XK_3270_CursorSelect   #xFD1C)
(define XK_3270_PrintScreen    #xFD1D)
(define XK_3270_Enter          #xFD1E)


;  Latin 1
;  Byte 3 = 0


(define XK_space               #x020)
(define XK_exclam              #x021)
(define XK_quotedbl            #x022)
(define XK_numbersign          #x023)
(define XK_dollar              #x024)
(define XK_percent             #x025)
(define XK_ampersand           #x026)
(define XK_apostrophe          #x027)
(define XK_quoteright          #x027)  ; deprecated
(define XK_parenleft           #x028)
(define XK_parenright          #x029)
(define XK_asterisk            #x02a)
(define XK_plus                #x02b)
(define XK_comma               #x02c)
(define XK_minus               #x02d)
(define XK_period              #x02e)
(define XK_slash               #x02f)
(define XK_0                   #x030)
(define XK_1                   #x031)
(define XK_2                   #x032)
(define XK_3                   #x033)
(define XK_4                   #x034)
(define XK_5                   #x035)
(define XK_6                   #x036)
(define XK_7                   #x037)
(define XK_8                   #x038)
(define XK_9                   #x039)
(define XK_colon               #x03a)
(define XK_semicolon           #x03b)
(define XK_less                #x03c)
(define XK_equal               #x03d)
(define XK_greater             #x03e)
(define XK_question            #x03f)
(define XK_at                  #x040)
(define XK_A                   #x041)
(define XK_B                   #x042)
(define XK_C                   #x043)
(define XK_D                   #x044)
(define XK_E                   #x045)
(define XK_F                   #x046)
(define XK_G                   #x047)
(define XK_H                   #x048)
(define XK_I                   #x049)
(define XK_J                   #x04a)
(define XK_K                   #x04b)
(define XK_L                   #x04c)
(define XK_M                   #x04d)
(define XK_N                   #x04e)
(define XK_O                   #x04f)
(define XK_P                   #x050)
(define XK_Q                   #x051)
(define XK_R                   #x052)
(define XK_S                   #x053)
(define XK_T                   #x054)
(define XK_U                   #x055)
(define XK_V                   #x056)
(define XK_W                   #x057)
(define XK_X                   #x058)
(define XK_Y                   #x059)
(define XK_Z                   #x05a)
(define XK_bracketleft         #x05b)
(define XK_backslash           #x05c)
(define XK_bracketright        #x05d)
(define XK_asciicircum         #x05e)
(define XK_underscore          #x05f)
(define XK_grave               #x060)
(define XK_quoteleft           #x060)  ; deprecated
(define XK_a                   #x061)
(define XK_b                   #x062)
(define XK_c                   #x063)
(define XK_d                   #x064)
(define XK_e                   #x065)
(define XK_f                   #x066)
(define XK_g                   #x067)
(define XK_h                   #x068)
(define XK_i                   #x069)
(define XK_j                   #x06a)
(define XK_k                   #x06b)
(define XK_l                   #x06c)
(define XK_m                   #x06d)
(define XK_n                   #x06e)
(define XK_o                   #x06f)
(define XK_p                   #x070)
(define XK_q                   #x071)
(define XK_r                   #x072)
(define XK_s                   #x073)
(define XK_t                   #x074)
(define XK_u                   #x075)
(define XK_v                   #x076)
(define XK_w                   #x077)
(define XK_x                   #x078)
(define XK_y                   #x079)
(define XK_z                   #x07a)
(define XK_braceleft           #x07b)
(define XK_bar                 #x07c)
(define XK_braceright          #x07d)
(define XK_asciitilde          #x07e)

(define XK_nobreakspace        #x0a0)
(define XK_exclamdown          #x0a1)
(define XK_cent        	       #x0a2)
(define XK_sterling            #x0a3)
(define XK_currency            #x0a4)
(define XK_yen                 #x0a5)
(define XK_brokenbar           #x0a6)
(define XK_section             #x0a7)
(define XK_diaeresis           #x0a8)
(define XK_copyright           #x0a9)
(define XK_ordfeminine         #x0aa)
(define XK_guillemotleft       #x0ab)  ; left angle quotation mark
(define XK_notsign             #x0ac)
(define XK_hyphen              #x0ad)
(define XK_registered          #x0ae)
(define XK_macron              #x0af)
(define XK_degree              #x0b0)
(define XK_plusminus           #x0b1)
(define XK_twosuperior         #x0b2)
(define XK_threesuperior       #x0b3)
(define XK_acute               #x0b4)
(define XK_mu                  #x0b5)
(define XK_paragraph           #x0b6)
(define XK_periodcentered      #x0b7)
(define XK_cedilla             #x0b8)
(define XK_onesuperior         #x0b9)
(define XK_masculine           #x0ba)
(define XK_guillemotright      #x0bb)  ; right angle quotation mark
(define XK_onequarter          #x0bc)
(define XK_onehalf             #x0bd)
(define XK_threequarters       #x0be)
(define XK_questiondown        #x0bf)
(define XK_Agrave              #x0c0)
(define XK_Aacute              #x0c1)
(define XK_Acircumflex         #x0c2)
(define XK_Atilde              #x0c3)
(define XK_Adiaeresis          #x0c4)
(define XK_Aring               #x0c5)
(define XK_AE                  #x0c6)
(define XK_Ccedilla            #x0c7)
(define XK_Egrave              #x0c8)
(define XK_Eacute              #x0c9)
(define XK_Ecircumflex         #x0ca)
(define XK_Ediaeresis          #x0cb)
(define XK_Igrave              #x0cc)
(define XK_Iacute              #x0cd)
(define XK_Icircumflex         #x0ce)
(define XK_Idiaeresis          #x0cf)
(define XK_ETH                 #x0d0)
(define XK_Eth                 #x0d0)  ; deprecated
(define XK_Ntilde              #x0d1)
(define XK_Ograve              #x0d2)
(define XK_Oacute              #x0d3)
(define XK_Ocircumflex         #x0d4)
(define XK_Otilde              #x0d5)
(define XK_Odiaeresis          #x0d6)
(define XK_multiply            #x0d7)
(define XK_Ooblique            #x0d8)
(define XK_Ugrave              #x0d9)
(define XK_Uacute              #x0da)
(define XK_Ucircumflex         #x0db)
(define XK_Udiaeresis          #x0dc)
(define XK_Yacute              #x0dd)
(define XK_THORN               #x0de)
(define XK_Thorn               #x0de)  ; deprecated
(define XK_ssharp              #x0df)
(define XK_agrave              #x0e0)
(define XK_aacute              #x0e1)
(define XK_acircumflex         #x0e2)
(define XK_atilde              #x0e3)
(define XK_adiaeresis          #x0e4)
(define XK_aring               #x0e5)
(define XK_ae                  #x0e6)
(define XK_ccedilla            #x0e7)
(define XK_egrave              #x0e8)
(define XK_eacute              #x0e9)
(define XK_ecircumflex         #x0ea)
(define XK_ediaeresis          #x0eb)
(define XK_igrave              #x0ec)
(define XK_iacute              #x0ed)
(define XK_icircumflex         #x0ee)
(define XK_idiaeresis          #x0ef)
(define XK_eth                 #x0f0)
(define XK_ntilde              #x0f1)
(define XK_ograve              #x0f2)
(define XK_oacute              #x0f3)
(define XK_ocircumflex         #x0f4)
(define XK_otilde              #x0f5)
(define XK_odiaeresis          #x0f6)
(define XK_division            #x0f7)
(define XK_oslash              #x0f8)
(define XK_ugrave              #x0f9)
(define XK_uacute              #x0fa)
(define XK_ucircumflex         #x0fb)
(define XK_udiaeresis          #x0fc)
(define XK_yacute              #x0fd)
(define XK_thorn               #x0fe)
(define XK_ydiaeresis          #x0ff)


;   Latin 2
;   Byte 3 = 1


(define XK_Aogonek             #x1a1)
(define XK_breve               #x1a2)
(define XK_Lstroke             #x1a3)
(define XK_Lcaron              #x1a5)
(define XK_Sacute              #x1a6)
(define XK_Scaron              #x1a9)
(define XK_Scedilla            #x1aa)
(define XK_Tcaron              #x1ab)
(define XK_Zacute              #x1ac)
(define XK_Zcaron              #x1ae)
(define XK_Zabovedot           #x1af)
(define XK_aogonek             #x1b1)
(define XK_ogonek              #x1b2)
(define XK_lstroke             #x1b3)
(define XK_lcaron              #x1b5)
(define XK_sacute              #x1b6)
(define XK_caron               #x1b7)
(define XK_scaron              #x1b9)
(define XK_scedilla            #x1ba)
(define XK_tcaron              #x1bb)
(define XK_zacute              #x1bc)
(define XK_doubleacute         #x1bd)
(define XK_zcaron              #x1be)
(define XK_zabovedot           #x1bf)
(define XK_Racute              #x1c0)
(define XK_Abreve              #x1c3)
(define XK_Lacute              #x1c5)
(define XK_Cacute              #x1c6)
(define XK_Ccaron              #x1c8)
(define XK_Eogonek             #x1ca)
(define XK_Ecaron              #x1cc)
(define XK_Dcaron              #x1cf)
(define XK_Dstroke             #x1d0)
(define XK_Nacute              #x1d1)
(define XK_Ncaron              #x1d2)
(define XK_Odoubleacute        #x1d5)
(define XK_Rcaron              #x1d8)
(define XK_Uring               #x1d9)
(define XK_Udoubleacute        #x1db)
(define XK_Tcedilla            #x1de)
(define XK_racute              #x1e0)
(define XK_abreve              #x1e3)
(define XK_lacute              #x1e5)
(define XK_cacute              #x1e6)
(define XK_ccaron              #x1e8)
(define XK_eogonek             #x1ea)
(define XK_ecaron              #x1ec)
(define XK_dcaron              #x1ef)
(define XK_dstroke             #x1f0)
(define XK_nacute              #x1f1)
(define XK_ncaron              #x1f2)
(define XK_odoubleacute        #x1f5)
(define XK_udoubleacute        #x1fb)
(define XK_rcaron              #x1f8)
(define XK_uring               #x1f9)
(define XK_tcedilla            #x1fe)
(define XK_abovedot            #x1ff)


;   Latin 3
;   Byte 3 = 2


(define XK_Hstroke             #x2a1)
(define XK_Hcircumflex         #x2a6)
(define XK_Iabovedot           #x2a9)
(define XK_Gbreve              #x2ab)
(define XK_Jcircumflex         #x2ac)
(define XK_hstroke             #x2b1)
(define XK_hcircumflex         #x2b6)
(define XK_idotless            #x2b9)
(define XK_gbreve              #x2bb)
(define XK_jcircumflex         #x2bc)
(define XK_Cabovedot           #x2c5)
(define XK_Ccircumflex         #x2c6)
(define XK_Gabovedot           #x2d5)
(define XK_Gcircumflex         #x2d8)
(define XK_Ubreve              #x2dd)
(define XK_Scircumflex         #x2de)
(define XK_cabovedot           #x2e5)
(define XK_ccircumflex         #x2e6)
(define XK_gabovedot           #x2f5)
(define XK_gcircumflex         #x2f8)
(define XK_ubreve              #x2fd)
(define XK_scircumflex         #x2fe)



;   Latin 4
;   Byte 3 = 3


(define XK_kra                 #x3a2)
(define XK_kappa               #x3a2)  ; deprecated
(define XK_Rcedilla            #x3a3)
(define XK_Itilde              #x3a5)
(define XK_Lcedilla            #x3a6)
(define XK_Emacron             #x3aa)
(define XK_Gcedilla            #x3ab)
(define XK_Tslash              #x3ac)
(define XK_rcedilla            #x3b3)
(define XK_itilde              #x3b5)
(define XK_lcedilla            #x3b6)
(define XK_emacron             #x3ba)
(define XK_gcedilla            #x3bb)
(define XK_tslash              #x3bc)
(define XK_ENG                 #x3bd)
(define XK_eng                 #x3bf)
(define XK_Amacron             #x3c0)
(define XK_Iogonek             #x3c7)
(define XK_Eabovedot           #x3cc)
(define XK_Imacron             #x3cf)
(define XK_Ncedilla            #x3d1)
(define XK_Omacron             #x3d2)
(define XK_Kcedilla            #x3d3)
(define XK_Uogonek             #x3d9)
(define XK_Utilde              #x3dd)
(define XK_Umacron             #x3de)
(define XK_amacron             #x3e0)
(define XK_iogonek             #x3e7)
(define XK_eabovedot           #x3ec)
(define XK_imacron             #x3ef)
(define XK_ncedilla            #x3f1)
(define XK_omacron             #x3f2)
(define XK_kcedilla            #x3f3)
(define XK_uogonek             #x3f9)
(define XK_utilde              #x3fd)
(define XK_umacron             #x3fe)


; Katakana
; Byte 3 = 4


(define XK_overline				       #x47e)
(define XK_kana_fullstop                               #x4a1)
(define XK_kana_openingbracket                         #x4a2)
(define XK_kana_closingbracket                         #x4a3)
(define XK_kana_comma                                  #x4a4)
(define XK_kana_conjunctive                            #x4a5)
(define XK_kana_middledot                              #x4a5)  ; deprecated
(define XK_kana_WO                                     #x4a6)
(define XK_kana_a                                      #x4a7)
(define XK_kana_i                                      #x4a8)
(define XK_kana_u                                      #x4a9)
(define XK_kana_e                                      #x4aa)
(define XK_kana_o                                      #x4ab)
(define XK_kana_ya                                     #x4ac)
(define XK_kana_yu                                     #x4ad)
(define XK_kana_yo                                     #x4ae)
(define XK_kana_tsu                                    #x4af)
(define XK_kana_tu                                     #x4af)  ; deprecated
(define XK_prolongedsound                              #x4b0)
(define XK_kana_A                                      #x4b1)
(define XK_kana_I                                      #x4b2)
(define XK_kana_U                                      #x4b3)
(define XK_kana_E                                      #x4b4)
(define XK_kana_O                                      #x4b5)
(define XK_kana_KA                                     #x4b6)
(define XK_kana_KI                                     #x4b7)
(define XK_kana_KU                                     #x4b8)
(define XK_kana_KE                                     #x4b9)
(define XK_kana_KO                                     #x4ba)
(define XK_kana_SA                                     #x4bb)
(define XK_kana_SHI                                    #x4bc)
(define XK_kana_SU                                     #x4bd)
(define XK_kana_SE                                     #x4be)
(define XK_kana_SO                                     #x4bf)
(define XK_kana_TA                                     #x4c0)
(define XK_kana_CHI                                    #x4c1)
(define XK_kana_TI                                     #x4c1)  ; deprecated
(define XK_kana_TSU                                    #x4c2)
(define XK_kana_TU                                     #x4c2)  ; deprecated
(define XK_kana_TE                                     #x4c3)
(define XK_kana_TO                                     #x4c4)
(define XK_kana_NA                                     #x4c5)
(define XK_kana_NI                                     #x4c6)
(define XK_kana_NU                                     #x4c7)
(define XK_kana_NE                                     #x4c8)
(define XK_kana_NO                                     #x4c9)
(define XK_kana_HA                                     #x4ca)
(define XK_kana_HI                                     #x4cb)
(define XK_kana_FU                                     #x4cc)
(define XK_kana_HU                                     #x4cc)  ; deprecated
(define XK_kana_HE                                     #x4cd)
(define XK_kana_HO                                     #x4ce)
(define XK_kana_MA                                     #x4cf)
(define XK_kana_MI                                     #x4d0)
(define XK_kana_MU                                     #x4d1)
(define XK_kana_ME                                     #x4d2)
(define XK_kana_MO                                     #x4d3)
(define XK_kana_YA                                     #x4d4)
(define XK_kana_YU                                     #x4d5)
(define XK_kana_YO                                     #x4d6)
(define XK_kana_RA                                     #x4d7)
(define XK_kana_RI                                     #x4d8)
(define XK_kana_RU                                     #x4d9)
(define XK_kana_RE                                     #x4da)
(define XK_kana_RO                                     #x4db)
(define XK_kana_WA                                     #x4dc)
(define XK_kana_N                                      #x4dd)
(define XK_voicedsound                                 #x4de)
(define XK_semivoicedsound                             #x4df)
(define XK_kana_switch          #xFF7E)  ; Alias for mode_switch


;  Arabic
;  Byte 3 = 5


(define XK_Arabic_comma                                #x5ac)
(define XK_Arabic_semicolon                            #x5bb)
(define XK_Arabic_question_mark                        #x5bf)
(define XK_Arabic_hamza                                #x5c1)
(define XK_Arabic_maddaonalef                          #x5c2)
(define XK_Arabic_hamzaonalef                          #x5c3)
(define XK_Arabic_hamzaonwaw                           #x5c4)
(define XK_Arabic_hamzaunderalef                       #x5c5)
(define XK_Arabic_hamzaonyeh                           #x5c6)
(define XK_Arabic_alef                                 #x5c7)
(define XK_Arabic_beh                                  #x5c8)
(define XK_Arabic_tehmarbuta                           #x5c9)
(define XK_Arabic_teh                                  #x5ca)
(define XK_Arabic_theh                                 #x5cb)
(define XK_Arabic_jeem                                 #x5cc)
(define XK_Arabic_hah                                  #x5cd)
(define XK_Arabic_khah                                 #x5ce)
(define XK_Arabic_dal                                  #x5cf)
(define XK_Arabic_thal                                 #x5d0)
(define XK_Arabic_ra                                   #x5d1)
(define XK_Arabic_zain                                 #x5d2)
(define XK_Arabic_seen                                 #x5d3)
(define XK_Arabic_sheen                                #x5d4)
(define XK_Arabic_sad                                  #x5d5)
(define XK_Arabic_dad                                  #x5d6)
(define XK_Arabic_tah                                  #x5d7)
(define XK_Arabic_zah                                  #x5d8)
(define XK_Arabic_ain                                  #x5d9)
(define XK_Arabic_ghain                                #x5da)
(define XK_Arabic_tatweel                              #x5e0)
(define XK_Arabic_feh                                  #x5e1)
(define XK_Arabic_qaf                                  #x5e2)
(define XK_Arabic_kaf                                  #x5e3)
(define XK_Arabic_lam                                  #x5e4)
(define XK_Arabic_meem                                 #x5e5)
(define XK_Arabic_noon                                 #x5e6)
(define XK_Arabic_ha                                   #x5e7)
(define XK_Arabic_heh                                  #x5e7)  ; deprecated
(define XK_Arabic_waw                                  #x5e8)
(define XK_Arabic_alefmaksura                          #x5e9)
(define XK_Arabic_yeh                                  #x5ea)
(define XK_Arabic_fathatan                             #x5eb)
(define XK_Arabic_dammatan                             #x5ec)
(define XK_Arabic_kasratan                             #x5ed)
(define XK_Arabic_fatha                                #x5ee)
(define XK_Arabic_damma                                #x5ef)
(define XK_Arabic_kasra                                #x5f0)
(define XK_Arabic_shadda                               #x5f1)
(define XK_Arabic_sukun                                #x5f2)
(define XK_Arabic_switch        #xFF7E)  ; Alias for mode_switch


; Cyrillic
; Byte 3 = 6

(define XK_Serbian_dje                                 #x6a1)
(define XK_Macedonia_gje                               #x6a2)
(define XK_Cyrillic_io                                 #x6a3)
(define XK_Ukrainian_ie                                #x6a4)
(define XK_Ukranian_je                                 #x6a4)  ; deprecated
(define XK_Macedonia_dse                               #x6a5)
(define XK_Ukrainian_i                                 #x6a6)
(define XK_Ukranian_i                                  #x6a6)  ; deprecated
(define XK_Ukrainian_yi                                #x6a7)
(define XK_Ukranian_yi                                 #x6a7)  ; deprecated
(define XK_Cyrillic_je                                 #x6a8)
(define XK_Serbian_je                                  #x6a8)  ; deprecated
(define XK_Cyrillic_lje                                #x6a9)
(define XK_Serbian_lje                                 #x6a9)  ; deprecated
(define XK_Cyrillic_nje                                #x6aa)
(define XK_Serbian_nje                                 #x6aa)  ; deprecated
(define XK_Serbian_tshe                                #x6ab)
(define XK_Macedonia_kje                               #x6ac)
(define XK_Byelorussian_shortu                         #x6ae)
(define XK_Cyrillic_dzhe                               #x6af)
(define XK_Serbian_dze                                 #x6af)  ; deprecated
(define XK_numerosign                                  #x6b0)
(define XK_Serbian_DJE                                 #x6b1)
(define XK_Macedonia_GJE                               #x6b2)
(define XK_Cyrillic_IO                                 #x6b3)
(define XK_Ukrainian_IE                                #x6b4)
(define XK_Ukranian_JE                                 #x6b4)  ; deprecated
(define XK_Macedonia_DSE                               #x6b5)
(define XK_Ukrainian_I                                 #x6b6)
(define XK_Ukranian_I                                  #x6b6)  ; deprecated
(define XK_Ukrainian_YI                                #x6b7)
(define XK_Ukranian_YI                                 #x6b7)  ; deprecated
(define XK_Cyrillic_JE                                 #x6b8)
(define XK_Serbian_JE                                  #x6b8)  ; deprecated
(define XK_Cyrillic_LJE                                #x6b9)
(define XK_Serbian_LJE                                 #x6b9)  ; deprecated
(define XK_Cyrillic_NJE                                #x6ba)
(define XK_Serbian_NJE                                 #x6ba)  ; deprecated
(define XK_Serbian_TSHE                                #x6bb)
(define XK_Macedonia_KJE                               #x6bc)
(define XK_Byelorussian_SHORTU                         #x6be)
(define XK_Cyrillic_DZHE                               #x6bf)
(define XK_Serbian_DZE                                 #x6bf)  ; deprecated
(define XK_Cyrillic_yu                                 #x6c0)
(define XK_Cyrillic_a                                  #x6c1)
(define XK_Cyrillic_be                                 #x6c2)
(define XK_Cyrillic_tse                                #x6c3)
(define XK_Cyrillic_de                                 #x6c4)
(define XK_Cyrillic_ie                                 #x6c5)
(define XK_Cyrillic_ef                                 #x6c6)
(define XK_Cyrillic_ghe                                #x6c7)
(define XK_Cyrillic_ha                                 #x6c8)
(define XK_Cyrillic_i                                  #x6c9)
(define XK_Cyrillic_shorti                             #x6ca)
(define XK_Cyrillic_ka                                 #x6cb)
(define XK_Cyrillic_el                                 #x6cc)
(define XK_Cyrillic_em                                 #x6cd)
(define XK_Cyrillic_en                                 #x6ce)
(define XK_Cyrillic_o                                  #x6cf)
(define XK_Cyrillic_pe                                 #x6d0)
(define XK_Cyrillic_ya                                 #x6d1)
(define XK_Cyrillic_er                                 #x6d2)
(define XK_Cyrillic_es                                 #x6d3)
(define XK_Cyrillic_te                                 #x6d4)
(define XK_Cyrillic_u                                  #x6d5)
(define XK_Cyrillic_zhe                                #x6d6)
(define XK_Cyrillic_ve                                 #x6d7)
(define XK_Cyrillic_softsign                           #x6d8)
(define XK_Cyrillic_yeru                               #x6d9)
(define XK_Cyrillic_ze                                 #x6da)
(define XK_Cyrillic_sha                                #x6db)
(define XK_Cyrillic_e                                  #x6dc)
(define XK_Cyrillic_shcha                              #x6dd)
(define XK_Cyrillic_che                                #x6de)
(define XK_Cyrillic_hardsign                           #x6df)
(define XK_Cyrillic_YU                                 #x6e0)
(define XK_Cyrillic_A                                  #x6e1)
(define XK_Cyrillic_BE                                 #x6e2)
(define XK_Cyrillic_TSE                                #x6e3)
(define XK_Cyrillic_DE                                 #x6e4)
(define XK_Cyrillic_IE                                 #x6e5)
(define XK_Cyrillic_EF                                 #x6e6)
(define XK_Cyrillic_GHE                                #x6e7)
(define XK_Cyrillic_HA                                 #x6e8)
(define XK_Cyrillic_I                                  #x6e9)
(define XK_Cyrillic_SHORTI                             #x6ea)
(define XK_Cyrillic_KA                                 #x6eb)
(define XK_Cyrillic_EL                                 #x6ec)
(define XK_Cyrillic_EM                                 #x6ed)
(define XK_Cyrillic_EN                                 #x6ee)
(define XK_Cyrillic_O                                  #x6ef)
(define XK_Cyrillic_PE                                 #x6f0)
(define XK_Cyrillic_YA                                 #x6f1)
(define XK_Cyrillic_ER                                 #x6f2)
(define XK_Cyrillic_ES                                 #x6f3)
(define XK_Cyrillic_TE                                 #x6f4)
(define XK_Cyrillic_U                                  #x6f5)
(define XK_Cyrillic_ZHE                                #x6f6)
(define XK_Cyrillic_VE                                 #x6f7)
(define XK_Cyrillic_SOFTSIGN                           #x6f8)
(define XK_Cyrillic_YERU                               #x6f9)
(define XK_Cyrillic_ZE                                 #x6fa)
(define XK_Cyrillic_SHA                                #x6fb)
(define XK_Cyrillic_E                                  #x6fc)
(define XK_Cyrillic_SHCHA                              #x6fd)
(define XK_Cyrillic_CHE                                #x6fe)
(define XK_Cyrillic_HARDSIGN                           #x6ff)


; Greek
; Byte 3 = 7

(define XK_Greek_ALPHAaccent                           #x7a1)
(define XK_Greek_EPSILONaccent                         #x7a2)
(define XK_Greek_ETAaccent                             #x7a3)
(define XK_Greek_IOTAaccent                            #x7a4)
(define XK_Greek_IOTAdiaeresis                         #x7a5)
(define XK_Greek_OMICRONaccent                         #x7a7)
(define XK_Greek_UPSILONaccent                         #x7a8)
(define XK_Greek_UPSILONdieresis                       #x7a9)
(define XK_Greek_OMEGAaccent                           #x7ab)
(define XK_Greek_accentdieresis                        #x7ae)
(define XK_Greek_horizbar                              #x7af)
(define XK_Greek_alphaaccent                           #x7b1)
(define XK_Greek_epsilonaccent                         #x7b2)
(define XK_Greek_etaaccent                             #x7b3)
(define XK_Greek_iotaaccent                            #x7b4)
(define XK_Greek_iotadieresis                          #x7b5)
(define XK_Greek_iotaaccentdieresis                    #x7b6)
(define XK_Greek_omicronaccent                         #x7b7)
(define XK_Greek_upsilonaccent                         #x7b8)
(define XK_Greek_upsilondieresis                       #x7b9)
(define XK_Greek_upsilonaccentdieresis                 #x7ba)
(define XK_Greek_omegaaccent                           #x7bb)
(define XK_Greek_ALPHA                                 #x7c1)
(define XK_Greek_BETA                                  #x7c2)
(define XK_Greek_GAMMA                                 #x7c3)
(define XK_Greek_DELTA                                 #x7c4)
(define XK_Greek_EPSILON                               #x7c5)
(define XK_Greek_ZETA                                  #x7c6)
(define XK_Greek_ETA                                   #x7c7)
(define XK_Greek_THETA                                 #x7c8)
(define XK_Greek_IOTA                                  #x7c9)
(define XK_Greek_KAPPA                                 #x7ca)
(define XK_Greek_LAMDA                                 #x7cb)
(define XK_Greek_LAMBDA                                #x7cb)
(define XK_Greek_MU                                    #x7cc)
(define XK_Greek_NU                                    #x7cd)
(define XK_Greek_XI                                    #x7ce)
(define XK_Greek_OMICRON                               #x7cf)
(define XK_Greek_PI                                    #x7d0)
(define XK_Greek_RHO                                   #x7d1)
(define XK_Greek_SIGMA                                 #x7d2)
(define XK_Greek_TAU                                   #x7d4)
(define XK_Greek_UPSILON                               #x7d5)
(define XK_Greek_PHI                                   #x7d6)
(define XK_Greek_CHI                                   #x7d7)
(define XK_Greek_PSI                                   #x7d8)
(define XK_Greek_OMEGA                                 #x7d9)
(define XK_Greek_alpha                                 #x7e1)
(define XK_Greek_beta                                  #x7e2)
(define XK_Greek_gamma                                 #x7e3)
(define XK_Greek_delta                                 #x7e4)
(define XK_Greek_epsilon                               #x7e5)
(define XK_Greek_zeta                                  #x7e6)
(define XK_Greek_eta                                   #x7e7)
(define XK_Greek_theta                                 #x7e8)
(define XK_Greek_iota                                  #x7e9)
(define XK_Greek_kappa                                 #x7ea)
(define XK_Greek_lamda                                 #x7eb)
(define XK_Greek_lambda                                #x7eb)
(define XK_Greek_mu                                    #x7ec)
(define XK_Greek_nu                                    #x7ed)
(define XK_Greek_xi                                    #x7ee)
(define XK_Greek_omicron                               #x7ef)
(define XK_Greek_pi                                    #x7f0)
(define XK_Greek_rho                                   #x7f1)
(define XK_Greek_sigma                                 #x7f2)
(define XK_Greek_finalsmallsigma                       #x7f3)
(define XK_Greek_tau                                   #x7f4)
(define XK_Greek_upsilon                               #x7f5)
(define XK_Greek_phi                                   #x7f6)
(define XK_Greek_chi                                   #x7f7)
(define XK_Greek_psi                                   #x7f8)
(define XK_Greek_omega                                 #x7f9)
(define XK_Greek_switch         #xFF7E)  ; Alias for mode_switch


; Technical
; Byte 3 = 8

(define XK_leftradical                                 #x8a1)
(define XK_topleftradical                              #x8a2)
(define XK_horizconnector                              #x8a3)
(define XK_topintegral                                 #x8a4)
(define XK_botintegral                                 #x8a5)
(define XK_vertconnector                               #x8a6)
(define XK_topleftsqbracket                            #x8a7)
(define XK_botleftsqbracket                            #x8a8)
(define XK_toprightsqbracket                           #x8a9)
(define XK_botrightsqbracket                           #x8aa)
(define XK_topleftparens                               #x8ab)
(define XK_botleftparens                               #x8ac)
(define XK_toprightparens                              #x8ad)
(define XK_botrightparens                              #x8ae)
(define XK_leftmiddlecurlybrace                        #x8af)
(define XK_rightmiddlecurlybrace                       #x8b0)
(define XK_topleftsummation                            #x8b1)
(define XK_botleftsummation                            #x8b2)
(define XK_topvertsummationconnector                   #x8b3)
(define XK_botvertsummationconnector                   #x8b4)
(define XK_toprightsummation                           #x8b5)
(define XK_botrightsummation                           #x8b6)
(define XK_rightmiddlesummation                        #x8b7)
(define XK_lessthanequal                               #x8bc)
(define XK_notequal                                    #x8bd)
(define XK_greaterthanequal                            #x8be)
(define XK_integral                                    #x8bf)
(define XK_therefore                                   #x8c0)
(define XK_variation                                   #x8c1)
(define XK_infinity                                    #x8c2)
(define XK_nabla                                       #x8c5)
(define XK_approximate                                 #x8c8)
(define XK_similarequal                                #x8c9)
(define XK_ifonlyif                                    #x8cd)
(define XK_implies                                     #x8ce)
(define XK_identical                                   #x8cf)
(define XK_radical                                     #x8d6)
(define XK_includedin                                  #x8da)
(define XK_includes                                    #x8db)
(define XK_intersection                                #x8dc)
(define XK_union                                       #x8dd)
(define XK_logicaland                                  #x8de)
(define XK_logicalor                                   #x8df)
(define XK_partialderivative                           #x8ef)
(define XK_function                                    #x8f6)
(define XK_leftarrow                                   #x8fb)
(define XK_uparrow                                     #x8fc)
(define XK_rightarrow                                  #x8fd)
(define XK_downarrow                                   #x8fe)


;  Special
;  Byte 3 = 9

(define XK_blank                                       #x9df)
(define XK_soliddiamond                                #x9e0)
(define XK_checkerboard                                #x9e1)
(define XK_ht                                          #x9e2)
(define XK_ff                                          #x9e3)
(define XK_cr                                          #x9e4)
(define XK_lf                                          #x9e5)
(define XK_nl                                          #x9e8)
(define XK_vt                                          #x9e9)
(define XK_lowrightcorner                              #x9ea)
(define XK_uprightcorner                               #x9eb)
(define XK_upleftcorner                                #x9ec)
(define XK_lowleftcorner                               #x9ed)
(define XK_crossinglines                               #x9ee)
(define XK_horizlinescan1                              #x9ef)
(define XK_horizlinescan3                              #x9f0)
(define XK_horizlinescan5                              #x9f1)
(define XK_horizlinescan7                              #x9f2)
(define XK_horizlinescan9                              #x9f3)
(define XK_leftt                                       #x9f4)
(define XK_rightt                                      #x9f5)
(define XK_bott                                        #x9f6)
(define XK_topt                                        #x9f7)
(define XK_vertbar                                     #x9f8)


;  Publishing
;  Byte 3 = a

(define XK_emspace                                     #xaa1)
(define XK_enspace                                     #xaa2)
(define XK_em3space                                    #xaa3)
(define XK_em4space                                    #xaa4)
(define XK_digitspace                                  #xaa5)
(define XK_punctspace                                  #xaa6)
(define XK_thinspace                                   #xaa7)
(define XK_hairspace                                   #xaa8)
(define XK_emdash                                      #xaa9)
(define XK_endash                                      #xaaa)
(define XK_signifblank                                 #xaac)
(define XK_ellipsis                                    #xaae)
(define XK_doubbaselinedot                             #xaaf)
(define XK_onethird                                    #xab0)
(define XK_twothirds                                   #xab1)
(define XK_onefifth                                    #xab2)
(define XK_twofifths                                   #xab3)
(define XK_threefifths                                 #xab4)
(define XK_fourfifths                                  #xab5)
(define XK_onesixth                                    #xab6)
(define XK_fivesixths                                  #xab7)
(define XK_careof                                      #xab8)
(define XK_figdash                                     #xabb)
(define XK_leftanglebracket                            #xabc)
(define XK_decimalpoint                                #xabd)
(define XK_rightanglebracket                           #xabe)
(define XK_marker                                      #xabf)
(define XK_oneeighth                                   #xac3)
(define XK_threeeighths                                #xac4)
(define XK_fiveeighths                                 #xac5)
(define XK_seveneighths                                #xac6)
(define XK_trademark                                   #xac9)
(define XK_signaturemark                               #xaca)
(define XK_trademarkincircle                           #xacb)
(define XK_leftopentriangle                            #xacc)
(define XK_rightopentriangle                           #xacd)
(define XK_emopencircle                                #xace)
(define XK_emopenrectangle                             #xacf)
(define XK_leftsinglequotemark                         #xad0)
(define XK_rightsinglequotemark                        #xad1)
(define XK_leftdoublequotemark                         #xad2)
(define XK_rightdoublequotemark                        #xad3)
(define XK_prescription                                #xad4)
(define XK_minutes                                     #xad6)
(define XK_seconds                                     #xad7)
(define XK_latincross                                  #xad9)
(define XK_hexagram                                    #xada)
(define XK_filledrectbullet                            #xadb)
(define XK_filledlefttribullet                         #xadc)
(define XK_filledrighttribullet                        #xadd)
(define XK_emfilledcircle                              #xade)
(define XK_emfilledrect                                #xadf)
(define XK_enopencircbullet                            #xae0)
(define XK_enopensquarebullet                          #xae1)
(define XK_openrectbullet                              #xae2)
(define XK_opentribulletup                             #xae3)
(define XK_opentribulletdown                           #xae4)
(define XK_openstar                                    #xae5)
(define XK_enfilledcircbullet                          #xae6)
(define XK_enfilledsqbullet                            #xae7)
(define XK_filledtribulletup                           #xae8)
(define XK_filledtribulletdown                         #xae9)
(define XK_leftpointer                                 #xaea)
(define XK_rightpointer                                #xaeb)
(define XK_club                                        #xaec)
(define XK_diamond                                     #xaed)
(define XK_heart                                       #xaee)
(define XK_maltesecross                                #xaf0)
(define XK_dagger                                      #xaf1)
(define XK_doubledagger                                #xaf2)
(define XK_checkmark                                   #xaf3)
(define XK_ballotcross                                 #xaf4)
(define XK_musicalsharp                                #xaf5)
(define XK_musicalflat                                 #xaf6)
(define XK_malesymbol                                  #xaf7)
(define XK_femalesymbol                                #xaf8)
(define XK_telephone                                   #xaf9)
(define XK_telephonerecorder                           #xafa)
(define XK_phonographcopyright                         #xafb)
(define XK_caret                                       #xafc)
(define XK_singlelowquotemark                          #xafd)
(define XK_doublelowquotemark                          #xafe)
(define XK_cursor                                      #xaff)


;  APL
;  Byte 3 = b

(define XK_leftcaret                                   #xba3)
(define XK_rightcaret                                  #xba6)
(define XK_downcaret                                   #xba8)
(define XK_upcaret                                     #xba9)
(define XK_overbar                                     #xbc0)
(define XK_downtack                                    #xbc2)
(define XK_upshoe                                      #xbc3)
(define XK_downstile                                   #xbc4)
(define XK_underbar                                    #xbc6)
(define XK_jot                                         #xbca)
(define XK_quad                                        #xbcc)
(define XK_uptack                                      #xbce)
(define XK_circle                                      #xbcf)
(define XK_upstile                                     #xbd3)
(define XK_downshoe                                    #xbd6)
(define XK_rightshoe                                   #xbd8)
(define XK_leftshoe                                    #xbda)
(define XK_lefttack                                    #xbdc)
(define XK_righttack                                   #xbfc)


; Hebrew
; Byte 3 = c

(define XK_hebrew_doublelowline                        #xcdf)
(define XK_hebrew_aleph                                #xce0)
(define XK_hebrew_bet                                  #xce1)
(define XK_hebrew_beth                                 #xce1)  ; deprecated
(define XK_hebrew_gimel                                #xce2)
(define XK_hebrew_gimmel                               #xce2)  ; deprecated
(define XK_hebrew_dalet                                #xce3)
(define XK_hebrew_daleth                               #xce3)  ; deprecated
(define XK_hebrew_he                                   #xce4)
(define XK_hebrew_waw                                  #xce5)
(define XK_hebrew_zain                                 #xce6)
(define XK_hebrew_zayin                                #xce6)  ; deprecated
(define XK_hebrew_chet                                 #xce7)
(define XK_hebrew_het                                  #xce7)  ; deprecated
(define XK_hebrew_tet                                  #xce8)
(define XK_hebrew_teth                                 #xce8)  ; deprecated
(define XK_hebrew_yod                                  #xce9)
(define XK_hebrew_finalkaph                            #xcea)
(define XK_hebrew_kaph                                 #xceb)
(define XK_hebrew_lamed                                #xcec)
(define XK_hebrew_finalmem                             #xced)
(define XK_hebrew_mem                                  #xcee)
(define XK_hebrew_finalnun                             #xcef)
(define XK_hebrew_nun                                  #xcf0)
(define XK_hebrew_samech                               #xcf1)
(define XK_hebrew_samekh                               #xcf1)  ; deprecated
(define XK_hebrew_ayin                                 #xcf2)
(define XK_hebrew_finalpe                              #xcf3)
(define XK_hebrew_pe                                   #xcf4)
(define XK_hebrew_finalzade                            #xcf5)
(define XK_hebrew_finalzadi                            #xcf5)  ; deprecated
(define XK_hebrew_zade                                 #xcf6)
(define XK_hebrew_zadi                                 #xcf6)  ; deprecated
(define XK_hebrew_qoph                                 #xcf7)
(define XK_hebrew_kuf                                  #xcf7)  ; deprecated
(define XK_hebrew_resh                                 #xcf8)
(define XK_hebrew_shin                                 #xcf9)
(define XK_hebrew_taw                                  #xcfa)
(define XK_hebrew_taf                                  #xcfa)  ; deprecated
(define XK_Hebrew_switch        #xFF7E)  ; Alias for mode_switch


; Thai
; Byte 3 = d

(define XK_Thai_kokai					#xda1)
(define XK_Thai_khokhai					#xda2)
(define XK_Thai_khokhuat				#xda3)
(define XK_Thai_khokhwai				#xda4)
(define XK_Thai_khokhon					#xda5)
(define XK_Thai_khorakhang			        #xda6)
(define XK_Thai_ngongu					#xda7)
(define XK_Thai_chochan					#xda8)
(define XK_Thai_choching				#xda9)
(define XK_Thai_chochang				#xdaa)
(define XK_Thai_soso					#xdab)
(define XK_Thai_chochoe					#xdac)
(define XK_Thai_yoying					#xdad)
(define XK_Thai_dochada					#xdae)
(define XK_Thai_topatak					#xdaf)
(define XK_Thai_thothan					#xdb0)
(define XK_Thai_thonangmontho			        #xdb1)
(define XK_Thai_thophuthao			        #xdb2)
(define XK_Thai_nonen					#xdb3)
(define XK_Thai_dodek					#xdb4)
(define XK_Thai_totao					#xdb5)
(define XK_Thai_thothung				#xdb6)
(define XK_Thai_thothahan				#xdb7)
(define XK_Thai_thothong	 			#xdb8)
(define XK_Thai_nonu					#xdb9)
(define XK_Thai_bobaimai				#xdba)
(define XK_Thai_popla					#xdbb)
(define XK_Thai_phophung				#xdbc)
(define XK_Thai_fofa					#xdbd)
(define XK_Thai_phophan					#xdbe)
(define XK_Thai_fofan					#xdbf)
(define XK_Thai_phosamphao			        #xdc0)
(define XK_Thai_moma					#xdc1)
(define XK_Thai_yoyak					#xdc2)
(define XK_Thai_rorua					#xdc3)
(define XK_Thai_ru					#xdc4)
(define XK_Thai_loling					#xdc5)
(define XK_Thai_lu					#xdc6)
(define XK_Thai_wowaen					#xdc7)
(define XK_Thai_sosala					#xdc8)
(define XK_Thai_sorusi					#xdc9)
(define XK_Thai_sosua					#xdca)
(define XK_Thai_hohip					#xdcb)
(define XK_Thai_lochula					#xdcc)
(define XK_Thai_oang					#xdcd)
(define XK_Thai_honokhuk				#xdce)
(define XK_Thai_paiyannoi				#xdcf)
(define XK_Thai_saraa					#xdd0)
(define XK_Thai_maihanakat				#xdd1)
(define XK_Thai_saraaa					#xdd2)
(define XK_Thai_saraam					#xdd3)
(define XK_Thai_sarai					#xdd4)
(define XK_Thai_saraii					#xdd5)
(define XK_Thai_saraue					#xdd6)
(define XK_Thai_sarauee					#xdd7)
(define XK_Thai_sarau					#xdd8)
(define XK_Thai_sarauu					#xdd9)
(define XK_Thai_phinthu					#xdda)
(define XK_Thai_maihanakat_maitho   			#xdde)
(define XK_Thai_baht					#xddf)
(define XK_Thai_sarae					#xde0)
(define XK_Thai_saraae					#xde1)
(define XK_Thai_sarao					#xde2)
(define XK_Thai_saraaimaimuan				#xde3)
(define XK_Thai_saraaimaimalai				#xde4)
(define XK_Thai_lakkhangyao				#xde5)
(define XK_Thai_maiyamok				#xde6)
(define XK_Thai_maitaikhu				#xde7)
(define XK_Thai_maiek					#xde8)
(define XK_Thai_maitho					#xde9)
(define XK_Thai_maitri					#xdea)
(define XK_Thai_maichattawa				#xdeb)
(define XK_Thai_thanthakhat				#xdec)
(define XK_Thai_nikhahit				#xded)
(define XK_Thai_leksun					#xdf0)
(define XK_Thai_leknung					#xdf1)
(define XK_Thai_leksong					#xdf2)
(define XK_Thai_leksam					#xdf3)
(define XK_Thai_leksi					#xdf4)
(define XK_Thai_lekha					#xdf5)
(define XK_Thai_lekhok					#xdf6)
(define XK_Thai_lekchet					#xdf7)
(define XK_Thai_lekpaet					#xdf8)
(define XK_Thai_lekkao					#xdf9)


;   Korean
;   Byte 3 = e

(define XK_Hangul		#xff31)  ; Hangul start/stop(toggle)
(define XK_Hangul_Start		#xff32)  ; Hangul start
(define XK_Hangul_End		#xff33)  ; Hangul end, English start
(define XK_Hangul_Hanja		#xff34)  ; Start Hangul->Hanja Conversion
(define XK_Hangul_Jamo		#xff35)  ; Hangul Jamo mode
(define XK_Hangul_Romaja	#xff36)  ; Hangul Romaja mode
(define XK_Hangul_Codeinput	#xff37)  ; Hangul code input mode
(define XK_Hangul_Jeonja	#xff38)  ; Jeonja mode
(define XK_Hangul_Banja		#xff39)  ; Banja mode
(define XK_Hangul_PreHanja	#xff3a)  ; Pre Hanja conversion
(define XK_Hangul_PostHanja	#xff3b)  ; Post Hanja conversion
(define XK_Hangul_SingleCandidate	#xff3c)  ; Single candidate
(define XK_Hangul_MultipleCandidate	#xff3d)  ; Multiple candidate
(define XK_Hangul_PreviousCandidate	#xff3e)  ; Previous candidate
(define XK_Hangul_Special	#xff3f)  ; Special symbols
(define XK_Hangul_switch	#xFF7E)  ; Alias for mode_switch

; Hangul Consonant Characters
(define XK_Hangul_Kiyeog				#xea1)
(define XK_Hangul_SsangKiyeog				#xea2)
(define XK_Hangul_KiyeogSios				#xea3)
(define XK_Hangul_Nieun					#xea4)
(define XK_Hangul_NieunJieuj				#xea5)
(define XK_Hangul_NieunHieuh				#xea6)
(define XK_Hangul_Dikeud				#xea7)
(define XK_Hangul_SsangDikeud				#xea8)
(define XK_Hangul_Rieul					#xea9)
(define XK_Hangul_RieulKiyeog				#xeaa)
(define XK_Hangul_RieulMieum				#xeab)
(define XK_Hangul_RieulPieub				#xeac)
(define XK_Hangul_RieulSios				#xead)
(define XK_Hangul_RieulTieut				#xeae)
(define XK_Hangul_RieulPhieuf				#xeaf)
(define XK_Hangul_RieulHieuh				#xeb0)
(define XK_Hangul_Mieum					#xeb1)
(define XK_Hangul_Pieub					#xeb2)
(define XK_Hangul_SsangPieub				#xeb3)
(define XK_Hangul_PieubSios				#xeb4)
(define XK_Hangul_Sios					#xeb5)
(define XK_Hangul_SsangSios				#xeb6)
(define XK_Hangul_Ieung					#xeb7)
(define XK_Hangul_Jieuj					#xeb8)
(define XK_Hangul_SsangJieuj				#xeb9)
(define XK_Hangul_Cieuc					#xeba)
(define XK_Hangul_Khieuq				#xebb)
(define XK_Hangul_Tieut					#xebc)
(define XK_Hangul_Phieuf				#xebd)
(define XK_Hangul_Hieuh					#xebe)

; Hangul Vowel Characters
(define XK_Hangul_A					#xebf)
(define XK_Hangul_AE					#xec0)
(define XK_Hangul_YA					#xec1)
(define XK_Hangul_YAE					#xec2)
(define XK_Hangul_EO					#xec3)
(define XK_Hangul_E					#xec4)
(define XK_Hangul_YEO					#xec5)
(define XK_Hangul_YE					#xec6)
(define XK_Hangul_O					#xec7)
(define XK_Hangul_WA					#xec8)
(define XK_Hangul_WAE					#xec9)
(define XK_Hangul_OE					#xeca)
(define XK_Hangul_YO					#xecb)
(define XK_Hangul_U					#xecc)
(define XK_Hangul_WEO					#xecd)
(define XK_Hangul_WE					#xece)
(define XK_Hangul_WI					#xecf)
(define XK_Hangul_YU					#xed0)
(define XK_Hangul_EU					#xed1)
(define XK_Hangul_YI					#xed2)
(define XK_Hangul_I					#xed3)

; Hangul syllable-final (JongSeong) Characters
(define XK_Hangul_J_Kiyeog				#xed4)
(define XK_Hangul_J_SsangKiyeog				#xed5)
(define XK_Hangul_J_KiyeogSios				#xed6)
(define XK_Hangul_J_Nieun				#xed7)
(define XK_Hangul_J_NieunJieuj				#xed8)
(define XK_Hangul_J_NieunHieuh				#xed9)
(define XK_Hangul_J_Dikeud				#xeda)
(define XK_Hangul_J_Rieul				#xedb)
(define XK_Hangul_J_RieulKiyeog				#xedc)
(define XK_Hangul_J_RieulMieum				#xedd)
(define XK_Hangul_J_RieulPieub				#xede)
(define XK_Hangul_J_RieulSios				#xedf)
(define XK_Hangul_J_RieulTieut				#xee0)
(define XK_Hangul_J_RieulPhieuf				#xee1)
(define XK_Hangul_J_RieulHieuh				#xee2)
(define XK_Hangul_J_Mieum				#xee3)
(define XK_Hangul_J_Pieub				#xee4)
(define XK_Hangul_J_PieubSios				#xee5)
(define XK_Hangul_J_Sios				#xee6)
(define XK_Hangul_J_SsangSios				#xee7)
(define XK_Hangul_J_Ieung				#xee8)
(define XK_Hangul_J_Jieuj				#xee9)
(define XK_Hangul_J_Cieuc				#xeea)
(define XK_Hangul_J_Khieuq				#xeeb)
(define XK_Hangul_J_Tieut				#xeec)
(define XK_Hangul_J_Phieuf				#xeed)
(define XK_Hangul_J_Hieuh				#xeee)

; Ancient Hangul Consonant Characters
(define XK_Hangul_RieulYeorinHieuh			#xeef)
(define XK_Hangul_SunkyeongeumMieum			#xef0)
(define XK_Hangul_SunkyeongeumPieub			#xef1)
(define XK_Hangul_PanSios				#xef2)
(define XK_Hangul_KkogjiDalrinIeung			#xef3)
(define XK_Hangul_SunkyeongeumPhieuf			#xef4)
(define XK_Hangul_YeorinHieuh				#xef5)

; Ancient Hangul Vowel Characters
(define XK_Hangul_AraeA					#xef6)
(define XK_Hangul_AraeAE				#xef7)

; Ancient Hangul syllable-final (JongSeong) Characters
(define XK_Hangul_J_PanSios				#xef8)
(define XK_Hangul_J_KkogjiDalrinIeung			#xef9)
(define XK_Hangul_J_YeorinHieuh				#xefa)

; Korean currency symbol
(define XK_Korean_Won					#xeff)
